"""
Agent Runtime 模块测试的共享配置和 Fixtures

提供测试用的标准化配置、Mock 对象和工具函数。
"""

import asyncio
import pytest
from datetime import datetime
from typing import Dict, Any, Generator, AsyncGenerator, List
from unittest.mock import Mock, AsyncMock

from ppio_sandbox.agent_runtime.runtime import (
    AgentRuntimeApp,
    AgentRuntimeServer,
    RuntimeConfig,
    AgentConfig,
    AgentMetadata,
    AgentSpec,
    RequestContext,
    InvocationRequest,
    InvocationResponse,
    PingResponse,
    PingStatus
)

from ppio_sandbox.agent_runtime.client import (
    AgentRuntimeClient,
    SandboxSession,
    AuthManager,
    TemplateManager,
    AgentTemplate,
    ClientConfig,
    SandboxConfig,
    SessionStatus,
    AuthenticationError,
    TemplateNotFoundError,
    SandboxCreationError,
    SessionNotFoundError,
    InvocationError,
    NetworkError
)


# =============================================================================
# 配置 Fixtures
# =============================================================================

@pytest.fixture
def runtime_config() -> RuntimeConfig:
    """提供测试用的运行时配置"""
    return RuntimeConfig(
        host="127.0.0.1",
        port=8888,  # 使用不同端口避免冲突
        debug=True,
        timeout=30,
        max_request_size=1024 * 512,  # 512KB
        cors_origins=["*"],
        enable_metrics=True,
        enable_middleware=True
    )


@pytest.fixture
def agent_config() -> AgentConfig:
    """提供测试用的 Agent 配置"""
    return AgentConfig(
        apiVersion="v1",
        kind="Agent",
        metadata=AgentMetadata(
            name="test-agent",
            version="1.0.0",
            author="test@example.com",
            description="Test Agent for unit testing",
            created="2024-01-01T00:00:00Z"
        ),
        spec=AgentSpec(
            entrypoint="test_agent.py"
        )
    )


@pytest.fixture
def sample_request() -> InvocationRequest:
    """提供示例调用请求"""
    return InvocationRequest(
        prompt="Test prompt",
        data={"key": "value"},
        sandbox_id="test-sandbox-123",
        timeout=30,
        stream=False,
        metadata={"test": True}
    )


@pytest.fixture
def sample_context() -> RequestContext:
    """提供示例请求上下文"""
    return RequestContext(
        sandbox_id="test-sandbox-123",
        request_id="test-request-456",
        headers={"Content-Type": "application/json"}
    )


# =============================================================================
# 应用和服务器 Fixtures
# =============================================================================

@pytest.fixture
def mock_app(runtime_config: RuntimeConfig) -> AgentRuntimeApp:
    """提供 Mock 的 AgentRuntimeApp"""
    app = AgentRuntimeApp(config=runtime_config)
    
    # 注册一个简单的测试 Agent
    @app.entrypoint
    def test_agent(request: dict, context: RequestContext) -> dict:
        return {
            "response": f"Processed: {request.get('prompt', '')}",
            "sandbox_id": context.sandbox_id,
            "request_id": context.request_id
        }
    
    # 注册健康检查
    @app.ping
    def test_ping() -> dict:
        return {"status": "healthy", "service": "test-agent"}
    
    return app


@pytest.fixture
def mock_streaming_app(runtime_config: RuntimeConfig) -> AgentRuntimeApp:
    """提供支持流式响应的 Mock AgentRuntimeApp"""
    app = AgentRuntimeApp(config=runtime_config)
    
    # 注册流式 Agent
    @app.entrypoint
    async def streaming_agent(request: dict, context: RequestContext) -> AsyncGenerator[str, None]:
        prompt = request.get('prompt', '')
        for i in range(3):
            await asyncio.sleep(0.01)  # 模拟处理时间
            yield f"Chunk {i}: {prompt}"
    
    return app


@pytest.fixture
def test_server(runtime_config: RuntimeConfig) -> AgentRuntimeServer:
    """提供测试服务器实例"""
    server = AgentRuntimeServer(runtime_config)
    
    # 设置测试用的入口点函数
    def test_entrypoint(request: dict) -> dict:
        return {"result": "test"}
    
    server.set_entrypoint_handler(test_entrypoint)
    return server


# =============================================================================
# Agent 函数 Fixtures
# =============================================================================

@pytest.fixture
def sync_agent_function():
    """同步 Agent 函数"""
    def agent(request: dict, context: RequestContext) -> dict:
        return {
            "response": "sync response",
            "request_data": request,
            "context_id": context.request_id
        }
    return agent


@pytest.fixture
def async_agent_function():
    """异步 Agent 函数"""
    async def agent(request: dict, context: RequestContext) -> dict:
        await asyncio.sleep(0.01)
        return {
            "response": "async response",
            "request_data": request,
            "context_id": context.request_id
        }
    return agent


@pytest.fixture
def sync_generator_agent():
    """同步生成器 Agent 函数"""
    def agent(request: dict) -> Generator[str, None, None]:
        for i in range(3):
            yield f"sync chunk {i}"
    return agent


@pytest.fixture
def async_generator_agent():
    """异步生成器 Agent 函数"""
    async def agent(request: dict) -> AsyncGenerator[str, None]:
        for i in range(3):
            await asyncio.sleep(0.01)
            yield f"async chunk {i}"
    return agent


@pytest.fixture
def error_agent_function():
    """抛出异常的 Agent 函数"""
    def agent(request: dict) -> dict:
        raise ValueError("Test error")
    return agent


# =============================================================================
# Mock HTTP 和网络 Fixtures
# =============================================================================

@pytest.fixture
def mock_http_client():
    """Mock HTTP 客户端"""
    client = AsyncMock()
    
    # 配置默认响应
    client.post.return_value.status_code = 200
    client.post.return_value.json.return_value = {
        "result": "mock response",
        "status": "success"
    }
    client.get.return_value.status_code = 200
    client.get.return_value.json.return_value = {
        "status": "healthy"
    }
    
    return client


# =============================================================================
# 测试数据 Fixtures
# =============================================================================

@pytest.fixture
def large_request_data() -> Dict[str, Any]:
    """大型请求数据（用于集成测试）"""
    return {
        "prompt": "Process this large dataset",
        "data": {
            "items": [{"id": i, "value": f"item_{i}"} for i in range(1000)]
        },
        "metadata": {
            "batch_size": 1000,
            "processing_mode": "bulk"
        }
    }


@pytest.fixture
def concurrent_requests() -> list:
    """并发请求数据集"""
    return [
        {"prompt": f"Request {i}", "data": {"id": i}}
        for i in range(100)
    ]


# =============================================================================
# 测试工具函数
# =============================================================================

@pytest.fixture
def assert_response_format():
    """验证响应格式的工具函数"""
    def _assert_format(response: dict, expected_keys: list = None):
        if expected_keys is None:
            expected_keys = ["result", "status", "duration"]
        
        assert isinstance(response, dict)
        for key in expected_keys:
            assert key in response
        
        if "status" in response:
            assert response["status"] in ["success", "error"]
        
        if "duration" in response:
            assert isinstance(response["duration"], (int, float))
            assert response["duration"] >= 0
    
    return _assert_format


@pytest.fixture
def assert_ping_format():
    """验证健康检查响应格式的工具函数"""
    def _assert_format(response: dict):
        assert isinstance(response, dict)
        assert "status" in response
        assert response["status"] in ["Healthy", "HealthyBusy"]
        
        if "timestamp" in response:
            assert isinstance(response["timestamp"], str)
        
        if "message" in response:
            assert isinstance(response["message"], str)
    
    return _assert_format


# =============================================================================
# Pytest 配置
# =============================================================================

def pytest_configure(config):
    """Pytest 配置"""
    config.addinivalue_line(
        "markers", "unit: 单元测试"
    )
    config.addinivalue_line(
        "markers", "integration: 集成测试"
    )
    config.addinivalue_line(
        "markers", "network: 需要网络的测试"
    )
    config.addinivalue_line(
        "markers", "slow: 慢速测试"
    )
    config.addinivalue_line(
        "markers", "compatibility: 兼容性测试"
    )


# =============================================================================
# 异步测试支持
# =============================================================================

# 移除自定义的 event_loop fixture，让 pytest-asyncio 使用默认的
# @pytest.fixture(scope="session")
# def event_loop():
#     """提供事件循环给整个测试会话"""
#     policy = asyncio.get_event_loop_policy()
#     loop = policy.new_event_loop()
#     yield loop
#     loop.close()


# =============================================================================
# 测试环境清理
# =============================================================================

@pytest.fixture(autouse=True)
def cleanup_context():
    """自动清理请求上下文"""
    from ppio_sandbox.agent_runtime.runtime.context import AgentRuntimeContext
    
    yield
    
    # 测试后清理上下文
    AgentRuntimeContext.clear_current_context()


@pytest.fixture
def temp_server_port():
    """提供临时服务器端口"""
    import socket
    
    # 找一个可用端口
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        s.bind(('', 0))
        port = s.getsockname()[1]
    
    return port


# =============================================================================
# Agent Runtime Client Fixtures
# =============================================================================

@pytest.fixture
def client_config() -> ClientConfig:
    """提供测试用的客户端配置"""
    return ClientConfig(
        base_url="https://api.test.ppio.ai",
        timeout=30,
        max_retries=3,
        retry_delay=1.0,
        max_connections=50,
        max_keepalive_connections=10,
        keepalive_expiry=30.0
    )


@pytest.fixture
def sandbox_config() -> SandboxConfig:
    """提供测试用的 Sandbox 配置"""
    return SandboxConfig(
        timeout_seconds=300,
        memory_limit="1Gi",
        cpu_limit="1",
        env_vars={"TEST_MODE": "true"},
        volumes=[],
        ports=[8080]
    )


@pytest.fixture
def test_api_key() -> str:
    """提供测试用的 API Key"""
    return "test-api-key-12345678"


@pytest.fixture
def auth_manager(test_api_key: str) -> AuthManager:
    """提供测试用的认证管理器"""
    return AuthManager(api_key=test_api_key)


@pytest.fixture
def sample_template() -> AgentTemplate:
    """提供示例 Agent 模板"""
    from datetime import datetime
    return AgentTemplate(
        template_id="test-template-123",
        name="test-agent",
        version="1.0.0",
        description="Test Agent Template",
        author="test@example.com",
        tags=["test", "ai", "chat"],
        created_at=datetime.now(),
        updated_at=datetime.now(),
        status="active",
        metadata={
            "agent": {
                "apiVersion": "v1",
                "kind": "Agent",
                "metadata": {
                    "name": "test-agent",
                    "version": "1.0.0",
                    "author": "test@example.com",
                    "description": "Test Agent Template",
                    "created": "2024-01-01T00:00:00Z"
                },
                "spec": {
                    "entrypoint": "agent.py",
                    "runtime": {
                        "timeout": 60,
                        "memory_limit": "1Gi",
                        "cpu_limit": "1"
                    },
                    "sandbox": {
                        "template_id": "test-template-123"
                    }
                },
                "status": {
                    "phase": "deployed",
                    "template_id": "test-template-123",
                    "last_deployed": "2024-01-01T00:00:00Z",
                    "build_id": "build-123"
                }
            }
        },
        size=1024 * 1024,  # 1MB
        build_time=30.5,
        dependencies=["python:3.11", "pydantic"],
        runtime_info={"python_version": "3.11", "packages": ["pydantic"]}
    )


@pytest.fixture
def sample_templates(sample_template: AgentTemplate) -> List[AgentTemplate]:
    """提供示例模板列表"""
    from datetime import datetime
    
    templates = [sample_template]
    
    # 添加更多测试模板
    for i in range(2, 5):
        template = AgentTemplate(
            template_id=f"test-template-{i}",
            name=f"test-agent-{i}",
            version=f"{i}.0.0",
            description=f"Test Agent Template {i}",
            author="test@example.com",
            tags=["test", "ai"] if i % 2 == 0 else ["test", "chat"],
            created_at=datetime.now(),
            updated_at=datetime.now(),
            status="active",
            metadata={},
            size=1024 * 512 * i,
            build_time=15.0 + i,
            dependencies=[],
            runtime_info={}
        )
        templates.append(template)
    
    return templates


@pytest.fixture
def mock_sandbox():
    """Mock Sandbox 实例"""
    sandbox = Mock()
    sandbox.id = "sandbox-test-123"
    sandbox.sandbox_id = "sandbox-test-123"
    sandbox.get_host.return_value = "test-sandbox.ppio.ai"
    
    # Mock 异步方法
    sandbox.pause = AsyncMock()
    sandbox.resume = AsyncMock()
    sandbox.close = AsyncMock()
    sandbox.kill = AsyncMock()
    
    return sandbox


@pytest.fixture
def mock_template_manager(sample_templates: List[AgentTemplate], auth_manager: AuthManager):
    """Mock TemplateManager"""
    manager = Mock(spec=TemplateManager)
    manager.auth_manager = auth_manager
    
    # Mock 异步方法
    manager.list_templates = AsyncMock(return_value=sample_templates)
    manager.get_template = AsyncMock(return_value=sample_templates[0])
    manager.template_exists = AsyncMock(return_value=True)
    manager.close = AsyncMock()
    
    return manager


@pytest.fixture
async def mock_sandbox_session(mock_sandbox, sample_template: AgentTemplate):
    """Mock SandboxSession"""
    session = Mock(spec=SandboxSession)
    session.template_id = sample_template.template_id
    session.sandbox = mock_sandbox
    session.sandbox_id = mock_sandbox.id
    session.session_id = mock_sandbox.id
    session.status = SessionStatus.ACTIVE
    session.created_at = datetime.now()
    session.last_activity = datetime.now()
    session.host_url = "https://test-sandbox.ppio.ai"
    session.is_active = True
    session.is_paused = False
    session.age_seconds = 0.0
    session.idle_seconds = 0.0
    
    # Mock 异步方法
    session.invoke = AsyncMock(return_value={"result": "test response"})
    session.ping = AsyncMock(return_value=PingResponse(status="healthy"))
    session.get_status = AsyncMock(return_value=SessionStatus.ACTIVE)
    session.pause = AsyncMock()
    session.resume = AsyncMock()
    session.refresh = AsyncMock()
    session.close = AsyncMock()
    
    return session


@pytest.fixture
async def mock_agent_client(
    client_config: ClientConfig,
    auth_manager: AuthManager,
    mock_template_manager,
    mock_sandbox_session
):
    """Mock AgentRuntimeClient"""
    client = Mock(spec=AgentRuntimeClient)
    client.config = client_config
    client.auth_manager = auth_manager
    client.template_manager = mock_template_manager
    client._sessions = {mock_sandbox_session.sandbox_id: mock_sandbox_session}
    client._closed = False
    
    # Mock 异步方法
    client.create_session = AsyncMock(return_value=mock_sandbox_session)
    client.get_session = AsyncMock(return_value=mock_sandbox_session)
    client.list_sessions = AsyncMock(return_value=[mock_sandbox_session])
    client.close_session = AsyncMock()
    client.close_all_sessions = AsyncMock()
    client.list_templates = AsyncMock(return_value=[])
    client.get_template = AsyncMock()
    client.invoke_agent = AsyncMock(return_value=InvocationResponse(
        result="test response",
        status="success",
        duration=0.5
    ))
    client.invoke_agent_stream = AsyncMock()
    client.close = AsyncMock()
    
    return client


# =============================================================================
# HTTP Mock Fixtures for Client Testing
# =============================================================================

@pytest.fixture
def mock_httpx_client():
    """Mock httpx.AsyncClient for HTTP requests"""
    client = AsyncMock()
    
    # Mock 成功的模板查询响应
    templates_response = Mock()
    templates_response.status_code = 200
    templates_response.json.return_value = {
        "templates": [
            {
                "id": "test-template-123",
                "name": "test-agent",
                "version": "1.0.0",
                "description": "Test Agent",
                "author": "test@example.com",
                "tags": ["test"],
                "created_at": "2024-01-01T00:00:00Z",
                "updated_at": "2024-01-01T00:00:00Z",
                "status": "active",
                "metadata": {},
                "size": 1024,
                "build_time": 30.0
            }
        ]
    }
    
    # Mock 成功的 Agent 调用响应
    invoke_response = Mock()
    invoke_response.status_code = 200
    invoke_response.json.return_value = {
        "result": "test response",
        "status": "success",
        "duration": 0.5
    }
    
    # Mock 健康检查响应
    ping_response = Mock()
    ping_response.status_code = 200
    ping_response.json.return_value = {
        "status": "healthy",
        "timestamp": "2024-01-01T00:00:00Z"
    }
    
    # 配置默认响应
    client.get.return_value = templates_response
    client.post.return_value = invoke_response
    client.aclose = AsyncMock()
    
    return client


@pytest.fixture
def mock_streaming_response():
    """Mock 流式响应"""
    async def mock_stream():
        chunks = ["chunk 1", "chunk 2", "chunk 3"]
        for chunk in chunks:
            yield chunk
    
    response = Mock()
    response.status_code = 200
    response.aiter_text = mock_stream
    
    return response


# =============================================================================
# Environment and Integration Test Fixtures
# =============================================================================

@pytest.fixture
def real_api_key():
    """真实的 API Key（从环境变量读取）"""
    import os
    return os.getenv("PPIO_API_KEY")


@pytest.fixture
async def real_client(real_api_key: str):
    """真实的 AgentRuntimeClient（用于集成测试）"""
    if not real_api_key:
        pytest.skip("PPIO_API_KEY not set - skipping integration test")
    
    client = AgentRuntimeClient(api_key=real_api_key)
    yield client
    await client.close()


# =============================================================================
# Concurrent Test Fixtures
# =============================================================================

@pytest.fixture
def concurrent_requests_data():
    """并发请求测试数据"""
    return [
        InvocationRequest(
            prompt=f"Test request {i}",
            data={"id": i, "batch": "concurrent_test"},
            metadata={"test_id": i}
        )
        for i in range(50)
    ]


@pytest.fixture
def large_request_data_client():
    """大型请求数据（客户端集成测试用）"""
    return InvocationRequest(
        prompt="Process this large dataset",
        data={
            "items": [
                {"id": i, "value": f"item_{i}", "data": "x" * 100}
                for i in range(1000)
            ]
        },
        metadata={
            "batch_size": 1000,
            "processing_mode": "bulk",
            "test": True
        },
        timeout=300
    )


# =============================================================================
# Test Utilities for Client
# =============================================================================

@pytest.fixture
def assert_client_response_format():
    """验证客户端响应格式的工具函数"""
    def _assert_format(response: InvocationResponse):
        assert isinstance(response, InvocationResponse)
        assert hasattr(response, 'result')
        assert hasattr(response, 'status')
        assert hasattr(response, 'duration')
        assert response.status in ["success", "error"]
        assert isinstance(response.duration, (int, float))
        assert response.duration >= 0
    
    return _assert_format


@pytest.fixture
def assert_template_format():
    """验证模板格式的工具函数"""
    def _assert_format(template: AgentTemplate):
        assert isinstance(template, AgentTemplate)
        assert template.template_id
        assert template.name
        assert template.version
        assert isinstance(template.tags, list)
        assert template.status
        assert isinstance(template.metadata, dict)
    
    return _assert_format


# =============================================================================
# Error Simulation Fixtures
# =============================================================================

@pytest.fixture
def mock_network_error_client():
    """模拟网络错误的 HTTP 客户端"""
    client = AsyncMock()
    client.get.side_effect = Exception("Network connection failed")
    client.post.side_effect = Exception("Network connection failed")
    return client


@pytest.fixture
def mock_auth_error_client():
    """模拟认证错误的 HTTP 客户端"""
    client = AsyncMock()
    
    error_response = Mock()
    error_response.status_code = 401
    error_response.text = "Unauthorized"
    
    client.get.return_value = error_response
    client.post.return_value = error_response
    
    return client


@pytest.fixture
def mock_template_not_found_client():
    """模拟模板不存在的 HTTP 客户端"""
    client = AsyncMock()
    
    # 列表查询返回空
    list_response = Mock()
    list_response.status_code = 200
    list_response.json.return_value = {"templates": []}
    
    # 单个查询返回 404
    get_response = Mock()
    get_response.status_code = 404
    get_response.text = "Template not found"
    
    client.get.side_effect = [list_response, get_response]
    
    return client



# PPIO Agent Runtime - HTTP Server Implementation
# Copyright (c) 2024 PPIO
# Licensed under the MIT License

"""HTTP server implementation for PPIO Agent Runtime based on Starlette."""

import asyncio
import inspect
import json
import logging
import time
import traceback
import uuid
from datetime import datetime
from typing import Any, Callable, Dict, List, Optional, Union, AsyncGenerator, Generator

from starlette.applications import Starlette
from starlette.middleware import Middleware
from starlette.middleware.cors import CORSMiddleware
from starlette.requests import Request
from starlette.responses import Response, JSONResponse, StreamingResponse
from starlette.routing import Route
import uvicorn

from .models import RuntimeConfig, InvocationRequest, InvocationResponse, PingResponse, PingStatus
from .context import RequestContext, AgentRuntimeContext


logger = logging.getLogger(__name__)


class AgentRuntimeServer:
    """Agent Runtime 服务器"""
    
    def __init__(self, config: RuntimeConfig):
        """初始化服务器
        
        Args:
            config: 运行时配置
        """
        self.config = config
        self._entrypoint_func: Optional[Callable] = None
        self._ping_func: Optional[Callable] = None
        self._middlewares: List[Callable] = []
        self._app: Optional[Starlette] = None
        
        self._setup_app()
    
    def set_entrypoint_handler(self, func: Callable) -> None:
        """设置入口点处理函数"""
        self._entrypoint_func = func
        logger.info(f"Entrypoint handler set: {func.__name__}")
    
    def set_ping_handler(self, func: Optional[Callable]) -> None:
        """设置健康检查处理函数"""
        self._ping_func = func
        if func:
            logger.info(f"Ping handler set: {func.__name__}")
    
    def add_middleware(self, middleware_func: Callable) -> None:
        """添加中间件"""
        self._middlewares.append(middleware_func)
        logger.info(f"Middleware added: {middleware_func.__name__}")
    
    def run(self, port: int, host: str) -> None:
        """启动服务器"""
        logger.info(f"Starting Agent Runtime server on {host}:{port}")
        uvicorn.run(
            self._app,
            host=host,
            port=port,
            log_level="info" if self.config.debug else "warning",
            access_log=self.config.debug
        )
    
    def _setup_app(self) -> None:
        """设置 Starlette 应用"""
        middleware = []
        
        # CORS 中间件
        if self.config.cors_origins:
            middleware.append(
                Middleware(
                    CORSMiddleware,
                    allow_origins=self.config.cors_origins,
                    allow_methods=["GET", "POST", "OPTIONS"],
                    allow_headers=["*"],
                )
            )
        
        # 路由
        routes = [
            Route("/", self._handle_root, methods=["GET"]),
            Route("/ping", self._handle_ping, methods=["GET"]),
            Route("/invocations", self._handle_invocations, methods=["POST"]),
        ]
        
        self._app = Starlette(
            debug=self.config.debug,
            routes=routes,
            middleware=middleware
        )
    
    async def _handle_root(self, request: Request) -> JSONResponse:
        """处理根端点"""
        return JSONResponse({
            "service": "PPIO Agent Runtime",
            "status": "running",
            "timestamp": datetime.now().isoformat(),
            "endpoints": {
                "invocations": "/invocations",
                "ping": "/ping"
            }
        })
    
    async def _handle_ping(self, request: Request) -> JSONResponse:
        """处理 /ping 端点"""
        try:
            if self._ping_func:
                # 调用自定义健康检查函数
                if inspect.iscoroutinefunction(self._ping_func):
                    result = await self._ping_func()
                else:
                    result = self._ping_func()
                
                if isinstance(result, dict):
                    response = PingResponse(**result)
                elif isinstance(result, PingResponse):
                    response = result
                else:
                    response = PingResponse(
                        status=PingStatus.HEALTHY,
                        message=str(result) if result else None,
                        timestamp=datetime.now().isoformat()
                    )
            else:
                response = PingResponse(
                    status=PingStatus.HEALTHY,
                    timestamp=datetime.now().isoformat()
                )
            
            return JSONResponse(response.dict())
            
        except Exception as e:
            logger.error(f"Ping function error: {e}")
            error_response = PingResponse(
                status=PingStatus.HEALTHY_BUSY,
                message=f"Health check error: {str(e)}",
                timestamp=datetime.now().isoformat()
            )
            return JSONResponse(error_response.dict(), status_code=500)
    
    async def _handle_invocations(self, request: Request) -> Response:
        """处理 /invocations 端点"""
        start_time = time.time()
        context = None
        
        try:
            # 检查请求大小
            content_length = request.headers.get("content-length")
            if content_length and int(content_length) > self.config.max_request_size:
                return JSONResponse(
                    {"error": f"Request too large: {content_length} bytes"},
                    status_code=413
                )
            
            # 解析请求体
            try:
                request_data = await request.json()
            except Exception as e:
                return JSONResponse(
                    {"error": f"Invalid JSON: {str(e)}"},
                    status_code=400
                )
            
            # 创建调用请求
            invoke_request = InvocationRequest(**request_data)
            
            # 创建请求上下文
            context = RequestContext(
                sandbox_id=invoke_request.sandbox_id,
                request_id=str(uuid.uuid4()),
                headers=dict(request.headers)
            )
            
            # 设置上下文
            AgentRuntimeContext.set_current_context(context)
            
            try:
                # 执行中间件
                for middleware in self._middlewares:
                    if inspect.iscoroutinefunction(middleware):
                        await middleware(request, self._call_next_placeholder)
                    else:
                        middleware(request, self._call_next_placeholder)
                
                # 执行 Agent 函数
                result = await self._execute_agent_function(invoke_request, context)
                
                # 处理流式响应
                if invoke_request.stream and self._is_streaming_result(result):
                    return await self._create_streaming_response(result)
                
                # 创建普通响应
                duration = time.time() - start_time
                response = InvocationResponse(
                    result=result,
                    status="success",
                    duration=duration,
                    metadata={"request_id": context.request_id}
                )
                
                return JSONResponse(response.dict())
                
            finally:
                # 清除上下文
                AgentRuntimeContext.clear_current_context()
                
        except Exception as e:
            logger.error(f"Invocation error: {e}\n{traceback.format_exc()}")
            duration = time.time() - start_time
            error_response = InvocationResponse(
                result=None,
                status="error",
                duration=duration,
                error=str(e),
                metadata={"request_id": context.request_id if context else None}
            )
            return JSONResponse(error_response.dict(), status_code=500)
    
    async def _execute_agent_function(self, request: InvocationRequest, context: RequestContext) -> Any:
        """执行 Agent 函数"""
        if not self._entrypoint_func:
            raise RuntimeError("No entrypoint function registered")
        
        # 准备函数参数
        func_signature = inspect.signature(self._entrypoint_func)
        params = list(func_signature.parameters.keys())
        
        # 根据函数签名决定传递参数
        if len(params) >= 2:
            # 函数接受 request 和 context 参数
            args = (request.dict(), context)
        else:
            # 函数只接受 request 参数
            args = (request.dict(),)
        
        # 执行函数
        if inspect.iscoroutinefunction(self._entrypoint_func):
            return await self._entrypoint_func(*args)
        else:
            # 在线程池中执行同步函数
            loop = asyncio.get_event_loop()
            return await loop.run_in_executor(None, self._entrypoint_func, *args)
    
    def _is_streaming_result(self, result: Any) -> bool:
        """检查结果是否为流式结果"""
        return (
            hasattr(result, '__iter__') or 
            hasattr(result, '__aiter__') or
            inspect.isgenerator(result) or
            inspect.isasyncgen(result)
        )
    
    async def _create_streaming_response(self, result: Any) -> StreamingResponse:
        """创建流式响应"""
        async def stream_generator():
            try:
                if inspect.isasyncgen(result):
                    # 异步生成器
                    async for chunk in result:
                        yield f"data: {json.dumps(str(chunk))}\n\n"
                elif inspect.isgenerator(result):
                    # 同步生成器
                    for chunk in result:
                        yield f"data: {json.dumps(str(chunk))}\n\n"
                elif hasattr(result, '__aiter__'):
                    # 异步迭代器
                    async for chunk in result:
                        yield f"data: {json.dumps(str(chunk))}\n\n"
                else:
                    # 普通迭代器
                    for chunk in result:
                        yield f"data: {json.dumps(str(chunk))}\n\n"
            except Exception as e:
                logger.error(f"Streaming error: {e}")
                yield f"data: {json.dumps({'error': str(e)})}\n\n"
        
        return StreamingResponse(
            stream_generator(),
            media_type="text/plain",
            headers={
                "Cache-Control": "no-cache",
                "Connection": "keep-alive",
            }
        )
    
    def _call_next_placeholder(self, request: Request):
        """中间件调用占位符"""
        # 这个函数在实际的中间件系统中会被替换
        pass



"""
数据模型单元测试

测试 Pydantic 数据模型的验证和序列化功能
"""

import pytest
from datetime import datetime
from typing import Dict, Any

from pydantic import ValidationError

from ppio_sandbox.agent_runtime.client.models import (
    SessionStatus,
    AgentTemplate,
    SandboxConfig,
    ClientConfig,
    InvocationRequest,
    InvocationResponse,
    PingResponse
)


class TestSessionStatus:
    """SessionStatus 枚举测试"""
    
    @pytest.mark.unit
    def test_session_status_values(self):
        """测试会话状态枚举值"""
        assert SessionStatus.ACTIVE == "active"
        assert SessionStatus.PAUSED == "paused"
        assert SessionStatus.INACTIVE == "inactive"
        assert SessionStatus.CLOSED == "closed"
        assert SessionStatus.ERROR == "error"
    
    @pytest.mark.unit
    def test_session_status_membership(self):
        """测试会话状态成员检查"""
        assert "active" in SessionStatus
        assert "invalid_status" not in SessionStatus
    
    @pytest.mark.unit
    def test_session_status_iteration(self):
        """测试会话状态迭代"""
        statuses = list(SessionStatus)
        expected = ["active", "paused", "inactive", "closed", "error"]
        assert len(statuses) == len(expected)
        for status in expected:
            assert status in statuses


class TestAgentTemplate:
    """AgentTemplate 模型测试"""
    
    @pytest.mark.unit
    def test_agent_template_valid(self, sample_template: AgentTemplate):
        """测试有效的 Agent 模板"""
        assert isinstance(sample_template, AgentTemplate)
        assert sample_template.template_id == "test-template-123"
        assert sample_template.name == "test-agent"
        assert sample_template.version == "1.0.0"
        assert isinstance(sample_template.tags, list)
        assert isinstance(sample_template.metadata, dict)
    
    @pytest.mark.unit
    def test_agent_template_minimal(self):
        """测试最小化的 Agent 模板"""
        template = AgentTemplate(
            template_id="minimal-template",
            name="minimal-agent",
            version="1.0.0",
            created_at=datetime.now(),
            updated_at=datetime.now(),
            status="active"
        )
        
        assert template.template_id == "minimal-template"
        assert template.description is None
        assert template.author is None
        assert template.tags == []
        assert template.metadata == {}
        assert template.size is None
        assert template.build_time is None
        assert template.dependencies == []
        assert template.runtime_info is None
    
    @pytest.mark.unit
    def test_agent_template_serialization(self, sample_template: AgentTemplate):
        """测试 Agent 模板序列化"""
        data = sample_template.dict()
        
        assert isinstance(data, dict)
        assert data["template_id"] == sample_template.template_id
        assert data["name"] == sample_template.name
        assert data["version"] == sample_template.version
        assert "created_at" in data
        assert "updated_at" in data
        assert "metadata" in data
    
    @pytest.mark.unit
    def test_agent_template_deserialization(self, sample_template: AgentTemplate):
        """测试 Agent 模板反序列化"""
        data = sample_template.dict()
        restored = AgentTemplate(**data)
        
        assert restored.template_id == sample_template.template_id
        assert restored.name == sample_template.name
        assert restored.version == sample_template.version
        assert restored.tags == sample_template.tags
        assert restored.metadata == sample_template.metadata
    
    @pytest.mark.unit
    def test_agent_template_missing_required_fields(self):
        """测试缺失必填字段时的验证错误"""
        with pytest.raises(ValidationError) as exc_info:
            AgentTemplate()
        
        errors = exc_info.value.errors()
        required_fields = {error["loc"][0] for error in errors}
        expected_fields = {"template_id", "name", "version", "created_at", "updated_at", "status"}
        
        assert expected_fields.issubset(required_fields)
    
    @pytest.mark.unit
    def test_agent_template_invalid_types(self):
        """测试无效类型时的验证错误"""
        with pytest.raises(ValidationError):
            AgentTemplate(
                template_id=123,  # 应该是字符串
                name="test",
                version="1.0.0",
                created_at="invalid-date",  # 应该是 datetime
                updated_at=datetime.now(),
                status="active"
            )


class TestSandboxConfig:
    """SandboxConfig 模型测试"""
    
    @pytest.mark.unit
    def test_sandbox_config_defaults(self):
        """测试 Sandbox 配置默认值"""
        config = SandboxConfig()
        
        assert config.timeout_seconds == 300
        assert config.memory_limit is None
        assert config.cpu_limit is None
        assert config.env_vars == {}
        assert config.volumes == []
        assert config.ports == [8080]
    
    @pytest.mark.unit
    def test_sandbox_config_custom(self, sandbox_config: SandboxConfig):
        """测试自定义 Sandbox 配置"""
        assert sandbox_config.timeout_seconds == 300
        assert sandbox_config.memory_limit == "1Gi"
        assert sandbox_config.cpu_limit == "1"
        assert sandbox_config.env_vars["TEST_MODE"] == "true"
        assert 8080 in sandbox_config.ports
    
    @pytest.mark.unit
    def test_sandbox_config_serialization(self, sandbox_config: SandboxConfig):
        """测试 Sandbox 配置序列化"""
        data = sandbox_config.dict()
        
        assert isinstance(data, dict)
        assert data["timeout_seconds"] == 300
        assert data["memory_limit"] == "1Gi"
        assert data["cpu_limit"] == "1"
        assert isinstance(data["env_vars"], dict)
        assert isinstance(data["volumes"], list)
        assert isinstance(data["ports"], list)


class TestClientConfig:
    """ClientConfig 模型测试"""
    
    @pytest.mark.unit
    def test_client_config_defaults(self):
        """测试客户端配置默认值"""
        config = ClientConfig()
        
        assert config.base_url is None
        assert config.timeout == 300
        assert config.max_retries == 3
        assert config.retry_delay == 1.0
        assert config.max_connections == 100
        assert config.max_keepalive_connections == 20
        assert config.keepalive_expiry == 30.0
    
    @pytest.mark.unit
    def test_client_config_custom(self, client_config: ClientConfig):
        """测试自定义客户端配置"""
        assert client_config.base_url == "https://api.test.ppio.ai"
        assert client_config.timeout == 30
        assert client_config.max_retries == 3
        assert client_config.max_connections == 50


class TestInvocationRequest:
    """InvocationRequest 模型测试"""
    
    @pytest.mark.unit
    def test_invocation_request_minimal(self):
        """测试最小化的调用请求"""
        request = InvocationRequest()
        
        assert request.prompt is None
        assert request.data is None
        assert request.sandbox_id is None
        assert request.timeout is None
        assert request.stream is False
        assert request.metadata is None
    
    @pytest.mark.unit
    def test_invocation_request_full(self, sample_request: InvocationRequest):
        """测试完整的调用请求"""
        assert sample_request.prompt == "Test prompt"
        assert sample_request.data == {"key": "value"}
        assert sample_request.sandbox_id == "test-sandbox-123"
        assert sample_request.timeout == 30
        assert sample_request.stream is False
        assert sample_request.metadata["test"] is True
    
    @pytest.mark.unit
    def test_invocation_request_session_id_compatibility(self, sample_request: InvocationRequest):
        """测试向后兼容的 session_id 属性"""
        assert sample_request.session_id == sample_request.sandbox_id
        assert sample_request.session_id == "test-sandbox-123"
    
    @pytest.mark.unit
    def test_invocation_request_streaming(self):
        """测试流式请求"""
        request = InvocationRequest(
            prompt="Stream this",
            stream=True,
            stream_options={"buffer_size": 1024}
        )
        
        assert request.stream is True
        assert request.stream_options["buffer_size"] == 1024
    
    @pytest.mark.unit
    def test_invocation_request_serialization(self, sample_request: InvocationRequest):
        """测试调用请求序列化"""
        data = sample_request.dict(exclude_unset=True)
        
        assert isinstance(data, dict)
        assert data["prompt"] == "Test prompt"
        assert data["sandbox_id"] == "test-sandbox-123"
        assert "session_id" not in data  # 属性不应该被序列化


class TestInvocationResponse:
    """InvocationResponse 模型测试"""
    
    @pytest.mark.unit
    def test_invocation_response_success(self):
        """测试成功的调用响应"""
        response = InvocationResponse(
            result="Success result",
            status="success",
            duration=1.5
        )
        
        assert response.result == "Success result"
        assert response.status == "success"
        assert response.duration == 1.5
        assert response.error is None
        assert response.error_type is None
    
    @pytest.mark.unit
    def test_invocation_response_error(self):
        """测试错误的调用响应"""
        response = InvocationResponse(
            result=None,
            status="error",
            duration=0.1,
            error="Something went wrong",
            error_type="ValueError"
        )
        
        assert response.result is None
        assert response.status == "error"
        assert response.error == "Something went wrong"
        assert response.error_type == "ValueError"
    
    @pytest.mark.unit
    def test_invocation_response_with_metrics(self):
        """测试包含指标信息的响应"""
        response = InvocationResponse(
            result="result",
            status="success",
            duration=2.0,
            processing_time=1.8,
            queue_time=0.2,
            tokens_used=150,
            cost=0.001
        )
        
        assert response.processing_time == 1.8
        assert response.queue_time == 0.2
        assert response.tokens_used == 150
        assert response.cost == 0.001
    
    @pytest.mark.unit
    def test_invocation_response_validation(self):
        """测试调用响应验证"""
        # 缺失必填字段
        with pytest.raises(ValidationError):
            InvocationResponse()
        
        # 无效的 duration - 实际上 Pydantic 允许负数，测试实际行为
        response = InvocationResponse(
            result="test",
            status="success",
            duration=-1.0  # 负数实际上是被接受的
        )
        assert response.duration == -1.0


class TestPingResponse:
    """PingResponse 模型测试"""
    
    @pytest.mark.unit
    def test_ping_response_minimal(self):
        """测试最小化的 ping 响应"""
        response = PingResponse(status="healthy")
        
        assert response.status == "healthy"
        assert response.message is None
        assert response.timestamp is None
    
    @pytest.mark.unit
    def test_ping_response_full(self):
        """测试完整的 ping 响应"""
        timestamp = datetime.now().isoformat()
        response = PingResponse(
            status="healthy",
            message="Service is running",
            timestamp=timestamp
        )
        
        assert response.status == "healthy"
        assert response.message == "Service is running"
        assert response.timestamp == timestamp
    
    @pytest.mark.unit
    def test_ping_response_validation(self):
        """测试 ping 响应验证"""
        # 缺失必填字段
        with pytest.raises(ValidationError):
            PingResponse()


class TestModelIntegration:
    """模型集成测试"""
    
    @pytest.mark.unit
    def test_nested_model_serialization(self, sample_template: AgentTemplate):
        """测试嵌套模型序列化"""
        # 创建包含模板的复杂请求
        request = InvocationRequest(
            prompt="Use this template",
            data={"template": sample_template.dict()},
            metadata={"template_id": sample_template.template_id}
        )
        
        data = request.dict()
        assert isinstance(data["data"]["template"], dict)
        assert data["data"]["template"]["template_id"] == sample_template.template_id
    
    @pytest.mark.unit
    def test_model_json_compatibility(self, sample_template: AgentTemplate):
        """测试模型 JSON 兼容性"""
        # 序列化为 JSON
        json_str = sample_template.json()
        assert isinstance(json_str, str)
        
        # 从 JSON 反序列化
        restored = AgentTemplate.parse_raw(json_str)
        assert restored.template_id == sample_template.template_id
        assert restored.name == sample_template.name
    
    @pytest.mark.unit
    def test_model_copy_and_update(self, sample_template: AgentTemplate):
        """测试模型复制和更新"""
        # 复制模型
        copied = sample_template.copy()
        assert copied.template_id == sample_template.template_id
        assert copied is not sample_template
        
        # 更新复制的模型
        updated = sample_template.copy(update={"version": "2.0.0"})
        assert updated.version == "2.0.0"
        assert sample_template.version == "1.0.0"  # 原模型不变
    
    @pytest.mark.unit
    def test_model_field_validation_edge_cases(self):
        """测试模型字段验证边界情况"""
        # 测试空字符串 - 实际上 Pydantic 允许空字符串，测试实际行为
        template = AgentTemplate(
            template_id="",  # 空字符串实际上是被接受的
            name="test",
            version="1.0.0",
            created_at=datetime.now(),
            updated_at=datetime.now(),
            status="active"
        )
        assert template.template_id == ""
        
        # 测试过长的字符串（如果有长度限制）
        very_long_string = "x" * 1000
        template = AgentTemplate(
            template_id=very_long_string,
            name="test",
            version="1.0.0",
            created_at=datetime.now(),
            updated_at=datetime.now(),
            status="active"
        )
        assert len(template.template_id) == 1000



"""
中间件集成测试

测试多个中间件的协同工作、执行顺序、异常传播、以及与 Agent 函数的交互。
"""

import asyncio
import json
import threading
import time
import requests
import pytest
from typing import Callable
from unittest.mock import Mock

from ppio_sandbox.agent_runtime.runtime.app import AgentRuntimeApp
from ppio_sandbox.agent_runtime.runtime.context import RequestContext
from ppio_sandbox.agent_runtime.runtime.models import RuntimeConfig


class TestMiddlewareBasicFlow:
    """中间件基础流程测试"""
    
    def setup_method(self, method):
        """每个测试方法前的设置"""
        port_map = {
            "test_single_middleware_e2e": 8940,
            "test_multiple_middleware_execution_order": 8941,
            "test_middleware_response_modification": 8942
        }
        self.test_port = port_map.get(method.__name__, 8944)
        self.base_url = f"http://localhost:{self.test_port}"
        self.server_thread = None
        self.execution_log = []  # 用于记录执行顺序
    
    def teardown_method(self):
        """清理"""
        pass
    
    def start_server_in_thread(self, app: AgentRuntimeApp):
        """在独立线程中启动服务器"""
        def run_server():
            try:
                app.run(port=self.test_port, host="127.0.0.1")
            except Exception as e:
                print(f"Server error: {e}")
        
        self.server_thread = threading.Thread(target=run_server, daemon=True)
        self.server_thread.start()
        self._wait_for_server_ready()
    
    def _wait_for_server_ready(self, timeout=5):
        """等待服务器准备就绪"""
        start_time = time.time()
        while time.time() - start_time < timeout:
            try:
                response = requests.get(f"{self.base_url}/ping", timeout=1)
                if response.status_code == 200:
                    return
            except requests.exceptions.RequestException:
                pass
            time.sleep(0.1)
        raise TimeoutError("Server did not start within timeout")
    
    @pytest.mark.integration
    def test_single_middleware_e2e(self):
        """测试单个中间件的端到端功能"""
        config = RuntimeConfig()
        app = AgentRuntimeApp(config=config)
        
        @app.entrypoint
        def test_agent(request: dict) -> dict:
            return {"response": "agent processed", "input": request}
        
        @app.middleware
        async def logging_middleware(request, call_next):
            # 记录请求
            start_time = time.time()
            response = await call_next(request)
            end_time = time.time()
            
            # 修改响应头
            response.headers["X-Processing-Time"] = str(end_time - start_time)
            response.headers["X-Middleware"] = "logging"
            return response
        
        self.start_server_in_thread(app)
        
        request_data = {
            "data": {"message": "middleware test"},
            "sandbox_id": "middleware-test"
        }
        
        response = requests.post(f"{self.base_url}/invocations", json=request_data)
        
        assert response.status_code == 200
        
        # 验证中间件添加的头部
        assert "X-Processing-Time" in response.headers
        assert response.headers["X-Middleware"] == "logging"
        
        # 验证响应内容
        data = response.json()
        assert data["status"] == "success"
        assert data["result"]["response"] == "agent processed"
    
    @pytest.mark.integration
    def test_multiple_middleware_execution_order(self):
        """测试多个中间件的执行顺序"""
        config = RuntimeConfig()
        app = AgentRuntimeApp(config=config)
        
        @app.entrypoint
        def test_agent(request: dict) -> dict:
            self.execution_log.append("agent_execution")
            return {"response": "processed"}
        
        @app.middleware
        async def middleware_1(request, call_next):
            self.execution_log.append("middleware_1_start")
            response = await call_next(request)
            self.execution_log.append("middleware_1_end")
            response.headers["X-Middleware-1"] = "executed"
            return response
        
        @app.middleware
        async def middleware_2(request, call_next):
            self.execution_log.append("middleware_2_start")
            response = await call_next(request)
            self.execution_log.append("middleware_2_end")
            response.headers["X-Middleware-2"] = "executed"
            return response
        
        @app.middleware
        async def middleware_3(request, call_next):
            self.execution_log.append("middleware_3_start")
            response = await call_next(request)
            self.execution_log.append("middleware_3_end")
            response.headers["X-Middleware-3"] = "executed"
            return response
        
        self.start_server_in_thread(app)
        
        request_data = {
            "data": {"message": "order test"},
            "sandbox_id": "order-test"
        }
        
        response = requests.post(f"{self.base_url}/invocations", json=request_data)
        
        assert response.status_code == 200
        
        # 验证执行顺序：中间件按注册顺序执行
        expected_order = [
            "middleware_1_start", "middleware_2_start", "middleware_3_start",
            "agent_execution",
            "middleware_3_end", "middleware_2_end", "middleware_1_end"
        ]
        assert self.execution_log == expected_order
        
        # 验证所有中间件都执行了
        assert response.headers["X-Middleware-1"] == "executed"
        assert response.headers["X-Middleware-2"] == "executed"
        assert response.headers["X-Middleware-3"] == "executed"
    
    @pytest.mark.integration
    def test_middleware_response_modification(self):
        """测试中间件修改响应"""
        config = RuntimeConfig()
        app = AgentRuntimeApp(config=config)
        
        @app.entrypoint
        def test_agent(request: dict) -> dict:
            return {"response": "original", "agent": "processed"}
        
        @app.middleware
        async def response_modifier_middleware(request, call_next):
            response = await call_next(request)
            
            # 添加自定义响应头
            response.headers["X-Response-Modified"] = "true"
            response.headers["X-Middleware-Version"] = "1.0"
            response.headers["X-Processing-Node"] = "middleware-node-1"
            
            return response
        
        self.start_server_in_thread(app)
        
        request_data = {
            "data": {"test": "response modification"},
            "sandbox_id": "response-mod-test"
        }
        
        response = requests.post(f"{self.base_url}/invocations", json=request_data)
        
        assert response.status_code == 200
        
        # 验证中间件添加的响应头
        assert response.headers["X-Response-Modified"] == "true"
        assert response.headers["X-Middleware-Version"] == "1.0"
        assert response.headers["X-Processing-Node"] == "middleware-node-1"
        
        # 验证原始响应内容未被破坏
        data = response.json()
        result = data["result"]
        assert result["response"] == "original"
        assert result["agent"] == "processed"


class TestMiddlewareErrorHandling:
    """中间件错误处理测试"""
    
    def setup_method(self, method):
        """设置"""
        port_map = {
            "test_middleware_exception_handling": 8950,
            "test_agent_error_through_middleware": 8951
        }
        self.test_port = port_map.get(method.__name__, 8952)
        self.base_url = f"http://localhost:{self.test_port}"
        self.server_thread = None
        self.error_log = []
    
    def teardown_method(self):
        """清理"""
        pass
    
    def start_server_in_thread(self, app: AgentRuntimeApp):
        """启动服务器"""
        def run_server():
            try:
                app.run(port=self.test_port, host="127.0.0.1")
            except Exception as e:
                print(f"Server error: {e}")
        
        self.server_thread = threading.Thread(target=run_server, daemon=True)
        self.server_thread.start()
        self._wait_for_server_ready()
    
    def _wait_for_server_ready(self, timeout=5):
        """等待服务器准备就绪"""
        start_time = time.time()
        while time.time() - start_time < timeout:
            try:
                response = requests.get(f"{self.base_url}/ping", timeout=1)
                if response.status_code == 200:
                    return
            except requests.exceptions.RequestException:
                pass
            time.sleep(0.1)
        raise TimeoutError("Server did not start within timeout")
    
    @pytest.mark.integration
    def test_middleware_exception_handling(self):
        """测试中间件异常处理"""
        config = RuntimeConfig()
        app = AgentRuntimeApp(config=config)
        
        @app.entrypoint
        def test_agent(request: dict) -> dict:
            return {"response": "should not reach here"}
        
        @app.middleware
        async def error_middleware(request, call_next):
            data = await request.json()
            if data.get("data", {}).get("should_error", False):
                raise ValueError("Middleware error")
            return await call_next(request)
        
        self.start_server_in_thread(app)
        
        # 测试正常请求
        normal_request = {
            "data": {"should_error": False},
            "sandbox_id": "normal-test"
        }
        
        response = requests.post(f"{self.base_url}/invocations", json=normal_request)
        assert response.status_code == 200
        
        # 测试中间件错误
        error_request = {
            "data": {"should_error": True},
            "sandbox_id": "error-test"
        }
        
        response = requests.post(f"{self.base_url}/invocations", json=error_request)
        assert response.status_code == 500
        
        data = response.json()
        assert data["status"] == "error"
        assert "Middleware error" in data["error"]
    
    @pytest.mark.integration
    def test_agent_error_through_middleware(self):
        """测试 Agent 错误通过中间件的传播"""
        config = RuntimeConfig()
        app = AgentRuntimeApp(config=config)
        
        @app.entrypoint
        def error_agent(request: dict) -> dict:
            data = request.get("data", {})
            if data.get("agent_error", False):
                raise ValueError("Agent processing error")
            return {"response": "success"}
        
        @app.middleware
        async def error_catching_middleware(request, call_next):
            try:
                response = await call_next(request)
                response.headers["X-Agent-Status"] = "success"
                return response
            except Exception as e:
                self.error_log.append(f"Caught agent error: {str(e)}")
                # 重新抛出异常让框架处理
                raise
        
        self.start_server_in_thread(app)
        
        # 测试 Agent 错误
        error_request = {
            "data": {"agent_error": True},
            "sandbox_id": "agent-error-test"
        }
        
        response = requests.post(f"{self.base_url}/invocations", json=error_request)
        assert response.status_code == 500
        
        data = response.json()
        assert data["status"] == "error"
        assert "Agent processing error" in data["error"]
        
        # 验证中间件捕获了错误
        assert any("Caught agent error" in log for log in self.error_log)


"""
端到端集成测试

测试 Agent Runtime Client 的完整流程
"""

import pytest
import asyncio
from unittest.mock import patch, Mock, AsyncMock

from ppio_sandbox.agent_runtime.client import AgentRuntimeClient
from ppio_sandbox.agent_runtime.client.models import InvocationRequest, SessionStatus
from ppio_sandbox.agent_runtime.client.exceptions import (
    TemplateNotFoundError,
    SandboxCreationError,
    InvocationError
)

from ..mocks.mock_api import setup_template_responses, setup_agent_responses
from ..mocks.mock_sandbox import MockAsyncSandbox
from ..mocks.test_fixtures import create_sample_template


class TestEndToEndBasicFlow:
    """端到端基础流程测试"""
    
    @pytest.mark.integration
    async def test_complete_agent_invocation_flow(self, real_api_key):
        """测试完整的 Agent 调用流程"""
        if not real_api_key:
            pytest.skip("需要真实 API Key 进行集成测试")
        
        async with AgentRuntimeClient(api_key=real_api_key) as client:
            # 1. 列出可用模板
            templates = await client.list_templates()
            assert len(templates) > 0
            
            # 选择第一个模板
            template = templates[0]
            
            # 2. 创建会话
            session = await client.create_session(template.template_id)
            assert session.status == SessionStatus.ACTIVE
            
            try:
                # 3. 调用 Agent
                response = await session.invoke("Hello, this is a test message")
                assert response is not None
                
                # 4. 检查会话状态
                status = await session.get_status()
                assert status in [SessionStatus.ACTIVE, SessionStatus.PAUSED]
                
                # 5. 测试健康检查
                ping_response = await session.ping()
                assert ping_response.status in ["healthy", "busy"]
                
            finally:
                # 6. 清理会话
                await session.close()
                assert session.status == SessionStatus.CLOSED
    
    @pytest.mark.integration
    @pytest.mark.network
    async def test_mock_complete_flow(self, test_api_key: str):
        """使用 Mock 的完整流程测试"""
        # 使用 aioresponses 模拟 HTTP 请求
        from aioresponses import aioresponses
        
        with aioresponses() as mock_http:
            # 设置模板响应
            setup_template_responses(mock_http)
            setup_agent_responses(mock_http)
            
            async with AgentRuntimeClient(api_key=test_api_key) as client:
                # Mock sandbox 创建
                with patch.object(client, '_create_sandbox_instance') as mock_create:
                    mock_sandbox = MockAsyncSandbox()
                    mock_create.return_value = mock_sandbox
                    
                    # 1. 获取模板
                    template = await client.get_template("test-template-123")
                    assert template.template_id == "test-template-123"
                    
                    # 2. 创建会话
                    session = await client.create_session(template.template_id)
                    assert session.template_id == template.template_id
                    
                    # 3. 执行调用
                    with patch.object(session, '_get_http_client') as mock_http_client:
                        mock_client = AsyncMock()
                        mock_response = Mock()
                        mock_response.status_code = 200
                        mock_response.json.return_value = {"result": "Mock response"}
                        mock_client.post.return_value = mock_response
                        mock_http_client.return_value = mock_client
                        
                        result = await session.invoke("Test prompt")
                        assert result["result"] == "Mock response"
                    
                    # 4. 关闭会话
                    await session.close()
                    assert session.status == SessionStatus.CLOSED
    
    @pytest.mark.integration
    async def test_convenience_method_flow(self, test_api_key: str):
        """测试便捷方法流程"""
        from aioresponses import aioresponses
        
        with aioresponses() as mock_http:
            setup_template_responses(mock_http)
            setup_agent_responses(mock_http)
            
            async with AgentRuntimeClient(api_key=test_api_key) as client:
                with patch.object(client, '_create_sandbox_instance') as mock_create:
                    mock_sandbox = MockAsyncSandbox()
                    mock_create.return_value = mock_sandbox
                    
                    # 使用便捷方法一次性完成调用
                    with patch.object(client, 'template_manager'):
                        client.template_manager.template_exists = AsyncMock(return_value=True)
                        
                        # Mock session invoke
                        with patch('ppio_sandbox.agent_runtime.client.session.SandboxSession') as mock_session_class:
                            mock_session = AsyncMock()
                            mock_session.sandbox_id = "mock-session"
                            mock_session.invoke.return_value = {"result": "convenience response"}
                            mock_session.close = AsyncMock()
                            mock_session_class.return_value = mock_session
                            
                            response = await client.invoke_agent(
                                template_id="test-template-123",
                                request="Test convenience call"
                            )
                            
                            assert response.status == "success"
                            assert response.result["result"] == "convenience response"


class TestEndToEndErrorHandling:
    """端到端错误处理测试"""
    
    @pytest.mark.integration
    async def test_template_not_found_flow(self, test_api_key: str):
        """测试模板不存在的错误流程"""
        from aioresponses import aioresponses
        
        with aioresponses() as mock_http:
            # 设置模板不存在的响应
            mock_http.get(
                "https://api.ppio.ai/v1/templates/agents/non-existent",
                status=404,
                payload={"error": "Template not found"}
            )
            
            async with AgentRuntimeClient(api_key=test_api_key) as client:
                # 尝试获取不存在的模板
                with pytest.raises(TemplateNotFoundError):
                    await client.get_template("non-existent")
                
                # 尝试创建基于不存在模板的会话
                with patch.object(client.template_manager, 'template_exists', return_value=False):
                    with pytest.raises(TemplateNotFoundError):
                        await client.create_session("non-existent")
    
    @pytest.mark.integration
    async def test_sandbox_creation_error_flow(self, test_api_key: str):
        """测试 Sandbox 创建错误流程"""
        async with AgentRuntimeClient(api_key=test_api_key) as client:
            with patch.object(client.template_manager, 'template_exists', return_value=True):
                with patch.object(client, '_create_sandbox_instance', side_effect=Exception("Sandbox creation failed")):
                    with pytest.raises(SandboxCreationError) as exc_info:
                        await client.create_session("test-template")
                    
                    assert "Failed to create sandbox session" in str(exc_info.value)
    
    @pytest.mark.integration
    async def test_invocation_error_flow(self, test_api_key: str):
        """测试调用错误流程"""
        async with AgentRuntimeClient(api_key=test_api_key) as client:
            with patch.object(client.template_manager, 'template_exists', return_value=True):
                with patch.object(client, '_create_sandbox_instance') as mock_create:
                    mock_sandbox = MockAsyncSandbox()
                    mock_create.return_value = mock_sandbox
                    
                    session = await client.create_session("test-template")
                    
                    # Mock HTTP 客户端返回错误
                    with patch.object(session, '_get_http_client') as mock_http_client:
                        mock_client = AsyncMock()
                        mock_response = Mock()
                        mock_response.status_code = 500
                        mock_response.text = "Internal Server Error"
                        mock_client.post.return_value = mock_response
                        mock_http_client.return_value = mock_client
                        
                        with pytest.raises(InvocationError) as exc_info:
                            await session.invoke("Test prompt")
                        
                        assert "HTTP request failed with status 500" in str(exc_info.value)
    
    @pytest.mark.integration
    async def test_network_error_recovery(self, test_api_key: str):
        """测试网络错误恢复"""
        from aioresponses import aioresponses
        
        async with AgentRuntimeClient(api_key=test_api_key) as client:
            # 第一次请求失败
            with aioresponses() as mock_http:
                mock_http.get(
                    "https://api.ppio.ai/v1/templates/agents",
                    exception=Exception("Network error")
                )
                
                with pytest.raises(Exception):
                    await client.list_templates()
            
            # 第二次请求成功
            with aioresponses() as mock_http:
                setup_template_responses(mock_http)
                
                templates = await client.list_templates()
                assert len(templates) > 0


class TestEndToEndStreamingFlow:
    """端到端流式处理测试"""
    
    @pytest.mark.integration
    async def test_streaming_invocation_flow(self, test_api_key: str):
        """测试流式调用流程"""
        async with AgentRuntimeClient(api_key=test_api_key) as client:
            with patch.object(client.template_manager, 'template_exists', return_value=True):
                with patch.object(client, '_create_sandbox_instance') as mock_create:
                    mock_sandbox = MockAsyncSandbox()
                    mock_create.return_value = mock_sandbox
                    
                    session = await client.create_session("test-template")
                    
                    # Mock 流式响应
                    async def mock_stream():
                        for i in range(5):
                            await asyncio.sleep(0.01)
                            yield f"Stream chunk {i}"
                    
                    with patch.object(session, '_get_http_client') as mock_http_client:
                        mock_client = AsyncMock()
                        mock_stream_response = Mock()
                        mock_stream_response.status_code = 200
                        mock_stream_response.aiter_text = mock_stream
                        mock_client.stream.return_value.__aenter__.return_value = mock_stream_response
                        mock_http_client.return_value = mock_client
                        
                        chunks = []
                        async for chunk in await session.invoke("Stream test", stream=True):
                            chunks.append(chunk)
                        
                        assert len(chunks) == 5
                        assert all("Stream chunk" in chunk for chunk in chunks)
    
    @pytest.mark.integration
    async def test_streaming_convenience_method(self, test_api_key: str):
        """测试流式便捷方法"""
        async with AgentRuntimeClient(api_key=test_api_key) as client:
            with patch.object(client.template_manager, 'template_exists', return_value=True):
                with patch.object(client, '_create_sandbox_instance') as mock_create:
                    mock_sandbox = MockAsyncSandbox()
                    mock_create.return_value = mock_sandbox
                    
                    # Mock session creation and streaming
                    with patch('ppio_sandbox.agent_runtime.client.session.SandboxSession') as mock_session_class:
                        async def mock_stream():
                            for i in range(3):
                                yield f"Convenience chunk {i}"
                        
                        mock_session = AsyncMock()
                        mock_session.sandbox_id = "convenience-session"
                        mock_session.invoke.return_value = mock_stream()
                        mock_session.close = AsyncMock()
                        mock_session_class.return_value = mock_session
                        
                        chunks = []
                        async for chunk in await client.invoke_agent_stream(
                            template_id="test-template",
                            request="Convenience stream test"
                        ):
                            chunks.append(chunk)
                        
                        assert len(chunks) == 3
                        assert all("Convenience chunk" in chunk for chunk in chunks)


class TestEndToEndSessionManagement:
    """端到端会话管理测试"""
    
    @pytest.mark.integration
    async def test_multiple_sessions_flow(self, test_api_key: str):
        """测试多会话管理流程"""
        async with AgentRuntimeClient(api_key=test_api_key) as client:
            with patch.object(client.template_manager, 'template_exists', return_value=True):
                with patch.object(client, '_create_sandbox_instance') as mock_create:
                    # 创建多个 mock sandbox
                    mock_sandboxes = [MockAsyncSandbox() for _ in range(3)]
                    mock_create.side_effect = mock_sandboxes
                    
                    # 创建多个会话
                    sessions = []
                    for i in range(3):
                        session = await client.create_session(f"template-{i}")
                        sessions.append(session)
                    
                    # 验证所有会话都存在
                    all_sessions = await client.list_sessions()
                    assert len(all_sessions) == 3
                    
                    # 逐个关闭会话
                    for session in sessions:
                        await client.close_session(session.sandbox_id)
                    
                    # 验证所有会话都被关闭
                    remaining_sessions = await client.list_sessions()
                    assert len(remaining_sessions) == 0
    
    @pytest.mark.integration
    async def test_session_lifecycle_flow(self, test_api_key: str):
        """测试会话生命周期流程"""
        async with AgentRuntimeClient(api_key=test_api_key) as client:
            with patch.object(client.template_manager, 'template_exists', return_value=True):
                with patch.object(client, '_create_sandbox_instance') as mock_create:
                    mock_sandbox = MockAsyncSandbox()
                    mock_create.return_value = mock_sandbox
                    
                    # 1. 创建会话
                    session = await client.create_session("test-template")
                    assert session.status == SessionStatus.ACTIVE
                    
                    # 2. 暂停会话
                    await session.pause()
                    assert session.status == SessionStatus.PAUSED
                    
                    # 3. 恢复会话
                    await session.resume()
                    assert session.status == SessionStatus.ACTIVE
                    
                    # 4. 执行一些操作
                    with patch.object(session, '_get_http_client') as mock_http_client:
                        mock_client = AsyncMock()
                        mock_response = Mock()
                        mock_response.status_code = 200
                        mock_response.json.return_value = {"result": "lifecycle test"}
                        mock_client.post.return_value = mock_response
                        mock_http_client.return_value = mock_client
                        
                        result = await session.invoke("Lifecycle test")
                        assert result["result"] == "lifecycle test"
                    
                    # 5. 关闭会话
                    await session.close()
                    assert session.status == SessionStatus.CLOSED
    
    @pytest.mark.integration
    async def test_concurrent_session_operations(self, test_api_key: str):
        """测试并发会话操作"""
        async with AgentRuntimeClient(api_key=test_api_key) as client:
            with patch.object(client.template_manager, 'template_exists', return_value=True):
                with patch.object(client, '_create_sandbox_instance') as mock_create:
                    mock_sandbox = MockAsyncSandbox()
                    mock_create.return_value = mock_sandbox
                    
                    session = await client.create_session("test-template")
                    
                    # Mock HTTP 客户端
                    with patch.object(session, '_get_http_client') as mock_http_client:
                        mock_client = AsyncMock()
                        mock_response = Mock()
                        mock_response.status_code = 200
                        mock_response.json.return_value = {"result": "concurrent"}
                        mock_client.post.return_value = mock_response
                        mock_client.get.return_value = mock_response
                        mock_http_client.return_value = mock_client
                        
                        # 并发执行多个操作
                        tasks = [
                            session.invoke(f"Concurrent request {i}")
                            for i in range(5)
                        ]
                        tasks.append(session.ping())
                        tasks.append(session.get_status())
                        
                        results = await asyncio.gather(*tasks, return_exceptions=True)
                        
                        # 验证结果
                        assert len(results) == 7
                        # 前5个是 invoke 结果
                        for i in range(5):
                            assert results[i]["result"] == "concurrent"


class TestEndToEndAuthentication:
    """端到端认证测试"""
    
    @pytest.mark.integration
    async def test_invalid_api_key_flow(self):
        """测试无效 API Key 流程"""
        from aioresponses import aioresponses
        
        with aioresponses() as mock_http:
            # 设置认证错误响应
            mock_http.get(
                "https://api.ppio.ai/v1/templates/agents",
                status=401,
                payload={"error": "Invalid API Key"}
            )
            
            async with AgentRuntimeClient(api_key="invalid-key") as client:
                from ppio_sandbox.agent_runtime.client.exceptions import AuthenticationError
                
                with pytest.raises(AuthenticationError):
                    await client.list_templates()
    
    @pytest.mark.integration
    async def test_api_key_rotation_flow(self, test_api_key: str):
        """测试 API Key 轮换流程"""
        from aioresponses import aioresponses
        
        client = AgentRuntimeClient(api_key="old-key")
        
        try:
            # 第一次请求失败（旧 key）
            with aioresponses() as mock_http:
                mock_http.get(
                    "https://api.ppio.ai/v1/templates/agents",
                    status=401,
                    payload={"error": "Invalid API Key"}
                )
                
                from ppio_sandbox.agent_runtime.client.exceptions import AuthenticationError
                with pytest.raises(AuthenticationError):
                    await client.list_templates()
            
            # 更新 API Key
            client.auth_manager.update_api_key(test_api_key)
            
            # 第二次请求成功（新 key）
            with aioresponses() as mock_http:
                setup_template_responses(mock_http)
                
                templates = await client.list_templates()
                assert len(templates) > 0
        
        finally:
            await client.close()


class TestEndToEndAdvanced:
    """端到端高级测试"""
    
    @pytest.mark.integration
    async def test_rapid_session_creation_cleanup(self, test_api_key: str):
        """测试快速会话创建和清理"""
        async with AgentRuntimeClient(api_key=test_api_key) as client:
            with patch.object(client.template_manager, 'template_exists', return_value=True):
                with patch.object(client, '_create_sandbox_instance') as mock_create:
                    
                    sessions = []
                    session_count = 10
                    
                    # 快速创建会话
                    for i in range(session_count):
                        mock_sandbox = MockAsyncSandbox()
                        mock_create.return_value = mock_sandbox
                        
                        session = await client.create_session(f"template-{i}")
                        sessions.append(session)
                    
                    assert len(client._sessions) == session_count
                    
                    # 批量关闭会话
                    await client.close_all_sessions()
                    assert len(client._sessions) == 0
    
    @pytest.mark.integration
    async def test_high_frequency_invocations(self, test_api_key: str):
        """测试高频调用"""
        async with AgentRuntimeClient(api_key=test_api_key) as client:
            with patch.object(client.template_manager, 'template_exists', return_value=True):
                with patch.object(client, '_create_sandbox_instance') as mock_create:
                    mock_sandbox = MockAsyncSandbox()
                    mock_create.return_value = mock_sandbox
                    
                    session = await client.create_session("test-template")
                    
                    # Mock HTTP 客户端
                    with patch.object(session, '_get_http_client') as mock_http_client:
                        mock_client = AsyncMock()
                        mock_response = Mock()
                        mock_response.status_code = 200
                        mock_response.json.return_value = {"result": "high-freq"}
                        mock_client.post.return_value = mock_response
                        mock_http_client.return_value = mock_client
                        
                        # 高频调用
                        invocation_count = 50
                        tasks = [
                            session.invoke(f"High frequency request {i}")
                            for i in range(invocation_count)
                        ]
                        
                        results = await asyncio.gather(*tasks)
                        
                        assert len(results) == invocation_count
                        assert all(r["result"] == "high-freq" for r in results)



"""
AgentRuntimeServer 单元测试

测试 AgentRuntimeServer 类的HTTP服务器功能，包括路由、中间件、请求处理等。
"""

import pytest
import asyncio
import json
from unittest.mock import Mock, AsyncMock, patch
from typing import Generator, AsyncGenerator

from starlette.applications import Starlette
from starlette.testclient import TestClient

from ppio_sandbox.agent_runtime.runtime.server import AgentRuntimeServer
from ppio_sandbox.agent_runtime.runtime.context import RequestContext, AgentRuntimeContext
from ppio_sandbox.agent_runtime.runtime.models import RuntimeConfig


class TestAgentRuntimeServerInitialization:
    """AgentRuntimeServer 初始化测试"""
    
    @pytest.mark.unit
    def test_default_initialization(self):
        """测试默认初始化"""
        config = RuntimeConfig()
        server = AgentRuntimeServer(config)
        
        assert server.config == config
        assert server._entrypoint_func is None
        assert server._ping_func is None
        assert server._middlewares == []
        assert server._app is not None
        assert isinstance(server._app, Starlette)
    
    @pytest.mark.unit
    def test_custom_config_initialization(self):
        """测试自定义配置初始化"""
        config = RuntimeConfig(
            host="127.0.0.1",
            port=9000,
            debug=True,
            max_request_size=2 * 1024 * 1024
        )
        
        server = AgentRuntimeServer(config)
        
        assert server.config.host == "127.0.0.1"
        assert server.config.port == 9000
        assert server.config.debug is True
        assert server.config.max_request_size == 2 * 1024 * 1024


class TestAgentRuntimeServerHandlerManagement:
    """AgentRuntimeServer 处理器管理测试"""
    
    @pytest.mark.unit
    def test_set_entrypoint_handler(self):
        """测试设置入口点处理器"""
        config = RuntimeConfig()
        server = AgentRuntimeServer(config)
        
        def test_agent(request: dict) -> dict:
            return {"response": "test", "input": request}
        
        server.set_entrypoint_handler(test_agent)
        
        assert server._entrypoint_func is test_agent
    
    @pytest.mark.unit
    def test_set_ping_handler(self):
        """测试设置 ping 处理器"""
        config = RuntimeConfig()
        server = AgentRuntimeServer(config)
        
        def test_ping() -> dict:
            return {"status": "healthy", "service": "test"}
        
        server.set_ping_handler(test_ping)
        
        assert server._ping_func is test_ping
    
    @pytest.mark.unit
    def test_add_middleware(self):
        """测试添加中间件"""
        config = RuntimeConfig()
        server = AgentRuntimeServer(config)
        
        async def test_middleware(request, call_next):
            response = await call_next(request)
            return response
        
        server.add_middleware(test_middleware)
        
        assert len(server._middlewares) == 1
        assert server._middlewares[0] is test_middleware


class TestAgentRuntimeServerRoutes:
    """AgentRuntimeServer 路由测试"""
    
    @pytest.mark.unit
    def test_root_endpoint(self):
        """测试根端点"""
        config = RuntimeConfig()
        server = AgentRuntimeServer(config)
        
        with TestClient(server._app) as client:
            response = client.get("/")
            
            assert response.status_code == 200
            data = response.json()
            assert data["service"] == "PPIO Agent Runtime"
            assert data["status"] == "running"
    
    @pytest.mark.unit
    def test_ping_endpoint_default(self):
        """测试默认 ping 端点"""
        config = RuntimeConfig()
        server = AgentRuntimeServer(config)
        
        with TestClient(server._app) as client:
            response = client.get("/ping")
            
            assert response.status_code == 200
            data = response.json()
            assert data["status"] == "Healthy"
            assert "timestamp" in data
    
    @pytest.mark.unit
    def test_invocations_endpoint_no_handler(self):
        """测试没有处理器的调用端点"""
        config = RuntimeConfig()
        server = AgentRuntimeServer(config)
        
        request_data = {
            "prompt": "test prompt",
            "sandbox_id": "test-sandbox"
        }
        
        with TestClient(server._app) as client:
            response = client.post("/invocations", json=request_data)
            
            assert response.status_code == 500
            data = response.json()
            assert data["status"] == "error"
            assert "No entrypoint function registered" in data["error"]
    
    @pytest.mark.unit
    def test_invocations_endpoint_with_handler(self):
        """测试带处理器的调用端点"""
        config = RuntimeConfig()
        server = AgentRuntimeServer(config)
        
        def test_agent(request: dict, context: RequestContext) -> dict:
            return {
                "response": f"Processed: {request.get('prompt', '')}",
                "sandbox_id": context.sandbox_id,
                "request_id": context.request_id
            }
        
        server.set_entrypoint_handler(test_agent)
        
        request_data = {
            "prompt": "test prompt",
            "sandbox_id": "test-sandbox",
            "stream": False
        }
        
        with TestClient(server._app) as client:
            response = client.post("/invocations", json=request_data)
            
            assert response.status_code == 200
            data = response.json()
            assert data["status"] == "success"
            assert data["result"]["response"] == "Processed: test prompt"
            assert data["result"]["sandbox_id"] == "test-sandbox"
            assert "duration" in data


class TestAgentRuntimeServerRequestProcessing:
    """AgentRuntimeServer 请求处理测试"""
    
    @pytest.mark.unit
    def test_json_request_parsing(self):
        """测试 JSON 请求解析"""
        config = RuntimeConfig()
        server = AgentRuntimeServer(config)
        
        def test_agent(request: dict) -> dict:
            return {"received": request}
        
        server.set_entrypoint_handler(test_agent)
        
        request_data = {
            "prompt": "test",
            "data": {"key": "value"},
            "sandbox_id": "test-sandbox"
        }
        
        with TestClient(server._app) as client:
            response = client.post("/invocations", json=request_data)
            
            assert response.status_code == 200
            data = response.json()
            assert data["result"]["received"]["prompt"] == "test"
            assert data["result"]["received"]["data"]["key"] == "value"
    
    @pytest.mark.unit
    def test_invalid_json_request(self):
        """测试无效 JSON 请求"""
        config = RuntimeConfig()
        server = AgentRuntimeServer(config)
        
        with TestClient(server._app) as client:
            response = client.post(
                "/invocations",
                content="invalid json",
                headers={"Content-Type": "application/json"}
            )
            
            assert response.status_code == 400
            data = response.json()
            assert "Invalid JSON" in data["error"]
    
    @pytest.mark.unit
    def test_async_agent_function(self):
        """测试异步 Agent 函数"""
        config = RuntimeConfig()
        server = AgentRuntimeServer(config)
        
        async def async_agent(request: dict) -> dict:
            await asyncio.sleep(0.01)  # 模拟异步操作
            return {"response": "async", "input": request.get("prompt")}
        
        server.set_entrypoint_handler(async_agent)
        
        request_data = {"prompt": "async test", "sandbox_id": "test"}
        
        with TestClient(server._app) as client:
            response = client.post("/invocations", json=request_data)
            
            assert response.status_code == 200
            data = response.json()
            assert data["result"]["response"] == "async"
            assert data["result"]["input"] == "async test"


class TestAgentRuntimeServerErrorHandling:
    """AgentRuntimeServer 错误处理测试"""
    
    @pytest.mark.unit
    def test_agent_function_exception(self):
        """测试 Agent 函数异常处理"""
        config = RuntimeConfig()
        server = AgentRuntimeServer(config)
        
        def error_agent(request: dict) -> dict:
            raise ValueError("Test error from agent")
        
        server.set_entrypoint_handler(error_agent)
        
        request_data = {"prompt": "test", "sandbox_id": "test"}
        
        with TestClient(server._app) as client:
            response = client.post("/invocations", json=request_data)
            
            assert response.status_code == 500
            data = response.json()
            assert data["status"] == "error"
            assert "Test error from agent" in data["error"]
            assert "duration" in data



"""
Sandbox 会话管理

管理单个 Sandbox 实例的生命周期和 Agent 调用
"""

import asyncio
import json
import time
from datetime import datetime
from typing import Any, AsyncIterator, Dict, Optional, Union
import httpx

from .exceptions import (
    InvocationError, 
    NetworkError, 
    SandboxOperationError, 
    SessionNotFoundError
)
from .models import InvocationRequest, PingResponse, SessionStatus


class SandboxSession:
    """Sandbox 会话管理"""
    
    def __init__(
        self,
        template_id: str,
        sandbox: Any,  # PPIO Sandbox 实例
        client: "AgentRuntimeClient"
    ):
        """初始化会话
        
        Args:
            template_id: 模板 ID
            sandbox: PPIO Sandbox 实例（一对一关系）
            client: Agent Runtime 客户端引用
        """
        self.template_id = template_id
        self.sandbox = sandbox
        self._client_ref = client  # 避免循环引用
        self.created_at = datetime.now()
        self.last_activity = datetime.now()
        self.status = SessionStatus.ACTIVE
        self._host_url: Optional[str] = None
        self._http_client: Optional[httpx.AsyncClient] = None
    
    async def _get_http_client(self) -> httpx.AsyncClient:
        """获取 HTTP 客户端"""
        if self._http_client is None:
            self._http_client = httpx.AsyncClient(
                timeout=httpx.Timeout(30.0),
                headers={"Content-Type": "application/json"}
            )
        return self._http_client
    
    async def _close_http_client(self):
        """关闭 HTTP 客户端"""
        if self._http_client:
            await self._http_client.aclose()
            self._http_client = None
    
    # === 核心调用方法 ===
    async def invoke(
        self,
        request: Union[InvocationRequest, Dict[str, Any], str],
        stream: bool = False
    ) -> Union[Dict[str, Any], AsyncIterator[str]]:
        """调用 Agent
        
        Args:
            request: 调用请求（支持多种格式）
            stream: 是否使用流式响应
            
        Returns:
            响应对象或流式迭代器
            
        Raises:
            InvocationError: 调用失败时抛出
            SessionNotFoundError: 会话不存在时抛出
            NetworkError: 网络错误时抛出
        """
        if self.status not in [SessionStatus.ACTIVE]:
            raise SessionNotFoundError(f"Session {self.sandbox_id} is not active (status: {self.status})")
        
        # 规范化请求格式
        if isinstance(request, str):
            request_data = InvocationRequest(prompt=request, stream=stream)
        elif isinstance(request, dict):
            request_data = InvocationRequest(**request)
            if stream:
                request_data.stream = stream
        elif isinstance(request, InvocationRequest):
            request_data = request
            if stream:
                request_data.stream = stream
        else:
            raise InvocationError("Invalid request format")
        
        # 设置 sandbox_id
        request_data.sandbox_id = self.sandbox_id
        
        try:
            self.last_activity = datetime.now()
            
            if request_data.stream:
                return await self._invoke_stream(request_data)
            else:
                return await self._invoke_sync(request_data)
                
        except Exception as e:
            if "connection" in str(e).lower() or "timeout" in str(e).lower():
                raise NetworkError(f"Network error during invocation: {str(e)}")
            else:
                raise InvocationError(f"Agent invocation failed: {str(e)}")
    
    async def _invoke_sync(self, request: InvocationRequest) -> Dict[str, Any]:
        """同步调用 Agent"""
        client = await self._get_http_client()
        
        response = await client.post(
            f"{self.host_url}/invocations",
            json=request.dict(exclude_unset=True),
            timeout=httpx.Timeout(request.timeout or 300)
        )
        
        if response.status_code != 200:
            raise InvocationError(f"Agent returned status {response.status_code}: {response.text}")
        
        return response.json()
    
    async def _invoke_stream(self, request: InvocationRequest) -> AsyncIterator[str]:
        """流式调用 Agent"""
        client = await self._get_http_client()
        
        async with client.stream(
            "POST",
            f"{self.host_url}/invocations", 
            json=request.dict(exclude_unset=True),
            timeout=httpx.Timeout(request.timeout or 300)
        ) as response:
            if response.status_code != 200:
                content = await response.aread()
                raise InvocationError(f"Agent returned status {response.status_code}: {content}")
            
            async for chunk in response.aiter_text():
                if chunk.strip():  # 跳过空行
                    yield chunk
    
    # === Sandbox 生命周期管理 ===
    async def pause(self) -> None:
        """暂停 Sandbox 实例
        
        暂停后：
        - Sandbox 进入休眠状态，保留内存状态
        - 停止 CPU 计算，节省资源
        - 可以通过 resume() 恢复执行
        
        Raises:
            SandboxOperationError: 暂停失败时抛出
        """
        try:
            if hasattr(self.sandbox, 'pause'):
                await self.sandbox.pause()
            else:
                # 如果 sandbox 没有 pause 方法，使用 API 调用
                await self._call_sandbox_api("pause")
            
            self.status = SessionStatus.PAUSED
            
        except Exception as e:
            raise SandboxOperationError(f"Failed to pause sandbox: {str(e)}")
    
    async def resume(self) -> None:
        """恢复 Sandbox 实例
        
        恢复后：
        - Sandbox 从暂停状态恢复
        - 保持之前的内存状态和上下文
        - 可以继续处理请求
        
        Raises:
            SandboxOperationError: 恢复失败时抛出
        """
        try:
            if hasattr(self.sandbox, 'resume'):
                await self.sandbox.resume()
            else:
                # 如果 sandbox 没有 resume 方法，使用 API 调用
                await self._call_sandbox_api("resume")
            
            self.status = SessionStatus.ACTIVE
            
        except Exception as e:
            raise SandboxOperationError(f"Failed to resume sandbox: {str(e)}")
    
    async def _call_sandbox_api(self, action: str) -> None:
        """调用 Sandbox API 执行操作"""
        # 这里需要根据实际的 Sandbox API 来实现
        # 暂时使用模拟实现
        pass
    
    # === 会话管理 ===
    async def ping(self) -> PingResponse:
        """健康检查
        
        Returns:
            健康检查响应
            
        Raises:
            NetworkError: 网络错误时抛出
            InvocationError: 检查失败时抛出
        """
        try:
            client = await self._get_http_client()
            
            response = await client.get(
                f"{self.host_url}/ping",
                timeout=httpx.Timeout(10.0)
            )
            
            if response.status_code == 200:
                data = response.json()
                return PingResponse(
                    status=data.get("status", "healthy"),
                    message=data.get("message"),
                    timestamp=data.get("timestamp", datetime.now().isoformat())
                )
            else:
                return PingResponse(
                    status="unhealthy",
                    message=f"HTTP {response.status_code}: {response.text}",
                    timestamp=datetime.now().isoformat()
                )
                
        except Exception as e:
            if "connection" in str(e).lower() or "timeout" in str(e).lower():
                raise NetworkError(f"Network error during ping: {str(e)}")
            else:
                raise InvocationError(f"Ping failed: {str(e)}")
    
    async def get_status(self) -> SessionStatus:
        """获取会话状态
        
        Returns:
            会话状态（ACTIVE, PAUSED, INACTIVE, CLOSED, ERROR）
        """
        # 可以添加实际的状态检查逻辑
        try:
            # 尝试 ping 来确认状态
            ping_response = await self.ping()
            if ping_response.status in ["healthy", "HealthyBusy"]:
                if self.status == SessionStatus.PAUSED:
                    return SessionStatus.PAUSED
                else:
                    return SessionStatus.ACTIVE
            else:
                return SessionStatus.ERROR
        except Exception:
            return SessionStatus.ERROR
    
    async def refresh(self) -> None:
        """刷新会话（重置超时时间）"""
        self.last_activity = datetime.now()
        # 可以添加实际的刷新逻辑，比如向 Sandbox 发送 keepalive 信号
    
    async def close(self) -> None:
        """关闭会话并销毁 Sandbox
        
        执行步骤：
        1. 停止 Agent 服务
        2. 销毁 Sandbox 实例
        3. 释放所有资源
        4. 更新会话状态为 CLOSED
        """
        try:
            # 关闭 HTTP 客户端
            await self._close_http_client()
            
            # 销毁 Sandbox 实例
            if hasattr(self.sandbox, 'close'):
                await self.sandbox.close()
            elif hasattr(self.sandbox, 'kill'):
                await self.sandbox.kill()
            
            self.status = SessionStatus.CLOSED
            
        except Exception as e:
            self.status = SessionStatus.ERROR
            raise SandboxOperationError(f"Failed to close session: {str(e)}")
    
    # === 属性 ===
    @property
    def host_url(self) -> str:
        """获取 Sandbox 主机 URL"""
        if not self._host_url:
            if self.sandbox and hasattr(self.sandbox, 'get_host'):
                # 使用实际的 Sandbox API
                host = self.sandbox.get_host(8080)
                self._host_url = f"https://{host}"
            else:
                # 模拟 URL（用于测试）
                self._host_url = f"https://session-{self.sandbox_id}.ppio.sandbox"
        return self._host_url
    
    @property
    def is_active(self) -> bool:
        """检查会话是否活跃"""
        return self.status == SessionStatus.ACTIVE
    
    @property
    def is_paused(self) -> bool:
        """检查会话是否已暂停"""
        return self.status == SessionStatus.PAUSED
    
    @property
    def sandbox_id(self) -> str:
        """获取 Sandbox 实例 ID（同时也是会话 ID）"""
        if hasattr(self.sandbox, 'id'):
            return self.sandbox.id
        elif hasattr(self.sandbox, 'sandbox_id'):
            return self.sandbox.sandbox_id
        else:
            return f"sandbox-{id(self.sandbox)}"
    
    @property
    def session_id(self) -> str:
        """获取会话 ID（等同于 sandbox_id）"""
        return self.sandbox_id
    
    @property
    def age_seconds(self) -> float:
        """获取会话存在时间（秒）"""
        return (datetime.now() - self.created_at).total_seconds()
    
    @property
    def idle_seconds(self) -> float:
        """获取会话空闲时间（秒）"""
        return (datetime.now() - self.last_activity).total_seconds()
    
    def __repr__(self) -> str:
        return f"SandboxSession(id={self.sandbox_id}, status={self.status}, template={self.template_id})"


"""
SandboxSession 单元测试

测试 Sandbox 会话管理功能
"""

import pytest
import asyncio
from datetime import datetime
from unittest.mock import Mock, AsyncMock, patch

from ppio_sandbox.agent_runtime.client.session import SandboxSession
from ppio_sandbox.agent_runtime.client.models import (
    InvocationRequest,
    SessionStatus,
    PingResponse
)
from ppio_sandbox.agent_runtime.client.exceptions import (
    InvocationError,
    NetworkError,
    SandboxOperationError,
    SessionNotFoundError
)

from ..mocks.mock_sandbox import (
    create_healthy_sandbox,
    create_paused_sandbox,
    create_error_sandbox,
    assert_sandbox_state,
    assert_operation_called
)
from ..mocks.test_fixtures import create_sample_request


class TestSandboxSessionInit:
    """SandboxSession 初始化测试"""
    
    @pytest.mark.unit
    def test_session_init(self, mock_sandbox, sample_template):
        """测试会话初始化"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        assert session.template_id == sample_template.template_id
        assert session.sandbox is mock_sandbox
        assert session._client_ref is mock_client
        assert session.status == SessionStatus.ACTIVE
        assert isinstance(session.created_at, datetime)
        assert isinstance(session.last_activity, datetime)
    
    @pytest.mark.unit
    def test_session_properties(self, mock_sandbox, sample_template):
        """测试会话属性"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        assert session.sandbox_id == mock_sandbox.id
        assert session.session_id == mock_sandbox.id  # 向后兼容
        assert session.host_url.startswith("https://")
        assert session.is_active is True
        assert session.is_paused is False
        assert session.age_seconds >= 0
        assert session.idle_seconds >= 0


class TestSandboxSessionInvoke:
    """SandboxSession 调用测试"""
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_invoke_success_dict(self, mock_sandbox, sample_template):
        """测试成功调用 - 字典格式请求"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        # Mock HTTP 响应
        mock_http_client = AsyncMock()
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {"result": "success response"}
        mock_http_client.post.return_value = mock_response
        
        with patch.object(session, '_get_http_client', return_value=mock_http_client):
            result = await session.invoke({"prompt": "test"})
        
        assert result == {"result": "success response"}
        assert mock_http_client.post.called
        
        # 验证最后活动时间被更新
        assert session.last_activity > session.created_at
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_invoke_success_string(self, mock_sandbox, sample_template):
        """测试成功调用 - 字符串格式请求"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        mock_http_client = AsyncMock()
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {"result": "string response"}
        mock_http_client.post.return_value = mock_response
        
        with patch.object(session, '_get_http_client', return_value=mock_http_client):
            result = await session.invoke("test prompt")
        
        assert result == {"result": "string response"}
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_invoke_success_invocation_request(self, mock_sandbox, sample_template, sample_request):
        """测试成功调用 - InvocationRequest 格式"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        mock_http_client = AsyncMock()
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {"result": "request response"}
        mock_http_client.post.return_value = mock_response
        
        with patch.object(session, '_get_http_client', return_value=mock_http_client):
            result = await session.invoke(sample_request)
        
        assert result == {"result": "request response"}
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_invoke_streaming(self, mock_sandbox, sample_template):
        """测试流式调用"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        # Mock 流式响应
        async def mock_stream():
            for chunk in ["chunk1", "chunk2", "chunk3"]:
                yield chunk
        
        mock_stream_response = Mock()
        mock_stream_response.status_code = 200
        mock_stream_response.aiter_text = mock_stream
        
        mock_http_client = AsyncMock()
        mock_http_client.stream.return_value.__aenter__.return_value = mock_stream_response
        
        with patch.object(session, '_get_http_client', return_value=mock_http_client):
            chunks = []
            async for chunk in await session.invoke("test", stream=True):
                chunks.append(chunk)
        
        assert chunks == ["chunk1", "chunk2", "chunk3"]
        assert mock_http_client.stream.called
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_invoke_session_not_active(self, mock_sandbox, sample_template):
        """测试会话非活跃状态调用"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        session.status = SessionStatus.CLOSED
        
        with pytest.raises(SessionNotFoundError) as exc_info:
            await session.invoke("test")
        
        assert "is not active" in str(exc_info.value)
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_invoke_http_error(self, mock_sandbox, sample_template):
        """测试 HTTP 错误"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        mock_http_client = AsyncMock()
        mock_response = Mock()
        mock_response.status_code = 500
        mock_response.text = "Internal Server Error"
        mock_http_client.post.return_value = mock_response
        
        with patch.object(session, '_get_http_client', return_value=mock_http_client):
            with pytest.raises(InvocationError) as exc_info:
                await session.invoke("test")
        
        assert "status 500" in str(exc_info.value)
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_invoke_network_error(self, mock_sandbox, sample_template):
        """测试网络错误"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        mock_http_client = AsyncMock()
        mock_http_client.post.side_effect = Exception("Connection timeout")
        
        with patch.object(session, '_get_http_client', return_value=mock_http_client):
            with pytest.raises(NetworkError) as exc_info:
                await session.invoke("test")
        
        assert "Network error" in str(exc_info.value)
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_invoke_invalid_request_format(self, mock_sandbox, sample_template):
        """测试无效请求格式"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        with pytest.raises(InvocationError) as exc_info:
            await session.invoke(123)  # 无效类型
        
        assert "Invalid request format" in str(exc_info.value)


class TestSandboxSessionLifecycle:
    """SandboxSession 生命周期测试"""
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_pause_success(self, sample_template):
        """测试成功暂停"""
        sandbox = create_healthy_sandbox()
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=sandbox,
            client=mock_client
        )
        
        await session.pause()
        
        assert session.status == SessionStatus.PAUSED
        assert_operation_called(sandbox, "pause", 1)
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_resume_success(self, sample_template):
        """测试成功恢复"""
        sandbox = create_paused_sandbox()
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=sandbox,
            client=mock_client
        )
        session.status = SessionStatus.PAUSED
        
        await session.resume()
        
        assert session.status == SessionStatus.ACTIVE
        assert_operation_called(sandbox, "resume", 1)
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_pause_error(self, sample_template):
        """测试暂停错误"""
        sandbox = create_error_sandbox()
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=sandbox,
            client=mock_client
        )
        
        with patch.object(sandbox, 'pause', side_effect=Exception("Pause failed")):
            with pytest.raises(SandboxOperationError) as exc_info:
                await session.pause()
        
        assert "Failed to pause sandbox" in str(exc_info.value)
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_close_success(self, sample_template):
        """测试成功关闭"""
        sandbox = create_healthy_sandbox()
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=sandbox,
            client=mock_client
        )
        
        await session.close()
        
        assert session.status == SessionStatus.CLOSED
        assert_operation_called(sandbox, "close", 1)
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_close_error(self, sample_template):
        """测试关闭错误"""
        sandbox = create_healthy_sandbox()
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=sandbox,
            client=mock_client
        )
        
        with patch.object(sandbox, 'close', side_effect=Exception("Close failed")):
            with pytest.raises(SandboxOperationError) as exc_info:
                await session.close()
        
        assert "Failed to close session" in str(exc_info.value)
        assert session.status == SessionStatus.ERROR


class TestSandboxSessionPing:
    """SandboxSession 健康检查测试"""
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_ping_success(self, mock_sandbox, sample_template):
        """测试成功的健康检查"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        mock_http_client = AsyncMock()
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
            "status": "healthy",
            "message": "Service running",
            "timestamp": "2024-01-01T00:00:00Z"
        }
        mock_http_client.get.return_value = mock_response
        
        with patch.object(session, '_get_http_client', return_value=mock_http_client):
            ping_response = await session.ping()
        
        assert isinstance(ping_response, PingResponse)
        assert ping_response.status == "healthy"
        assert ping_response.message == "Service running"
        assert ping_response.timestamp == "2024-01-01T00:00:00Z"
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_ping_unhealthy(self, mock_sandbox, sample_template):
        """测试不健康状态"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        mock_http_client = AsyncMock()
        mock_response = Mock()
        mock_response.status_code = 500
        mock_response.text = "Internal Error"
        mock_http_client.get.return_value = mock_response
        
        with patch.object(session, '_get_http_client', return_value=mock_http_client):
            ping_response = await session.ping()
        
        assert ping_response.status == "unhealthy"
        assert "HTTP 500" in ping_response.message
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_ping_network_error(self, mock_sandbox, sample_template):
        """测试 ping 网络错误"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        mock_http_client = AsyncMock()
        mock_http_client.get.side_effect = Exception("Connection failed")
        
        with patch.object(session, '_get_http_client', return_value=mock_http_client):
            with pytest.raises(NetworkError) as exc_info:
                await session.ping()
        
        assert "Network error during ping" in str(exc_info.value)


class TestSandboxSessionStatus:
    """SandboxSession 状态管理测试"""
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_get_status_active(self, mock_sandbox, sample_template):
        """测试获取活跃状态"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        # Mock ping 返回健康状态
        with patch.object(session, 'ping', return_value=PingResponse(status="healthy")):
            status = await session.get_status()
        
        assert status == SessionStatus.ACTIVE
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_get_status_paused(self, mock_sandbox, sample_template):
        """测试获取暂停状态"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        session.status = SessionStatus.PAUSED
        
        # Mock ping 返回健康状态，但会话处于暂停状态
        with patch.object(session, 'ping', return_value=PingResponse(status="healthy")):
            status = await session.get_status()
        
        assert status == SessionStatus.PAUSED
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_get_status_error(self, mock_sandbox, sample_template):
        """测试获取错误状态"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        # Mock ping 抛出异常
        with patch.object(session, 'ping', side_effect=Exception("Ping failed")):
            status = await session.get_status()
        
        assert status == SessionStatus.ERROR
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_refresh(self, mock_sandbox, sample_template):
        """测试刷新会话"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        old_activity = session.last_activity
        await asyncio.sleep(0.01)  # 确保时间差
        
        await session.refresh()
        
        assert session.last_activity > old_activity


class TestSandboxSessionProperties:
    """SandboxSession 属性测试"""
    
    @pytest.mark.unit
    def test_host_url_with_get_host(self, sample_template):
        """测试有 get_host 方法的 host_url"""
        sandbox = Mock()
        sandbox.id = "test-sandbox-123"
        sandbox.get_host.return_value = "test-host.example.com"
        
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=sandbox,
            client=mock_client
        )
        
        assert session.host_url == "https://test-host.example.com"
        sandbox.get_host.assert_called_with(8080)
    
    @pytest.mark.unit
    def test_host_url_without_get_host(self, sample_template):
        """测试没有 get_host 方法的 host_url"""
        sandbox = Mock()
        sandbox.id = "test-sandbox-123"
        del sandbox.get_host  # 删除 get_host 方法
        
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=sandbox,
            client=mock_client
        )
        
        assert session.host_url.startswith("https://session-")
        assert "test-sandbox-123" in session.host_url
    
    @pytest.mark.unit
    def test_sandbox_id_variants(self, sample_template):
        """测试不同的 sandbox_id 获取方式"""
        # 有 id 属性
        sandbox1 = Mock()
        sandbox1.id = "sandbox-with-id"
        
        session1 = SandboxSession(sample_template.template_id, sandbox1, Mock())
        assert session1.sandbox_id == "sandbox-with-id"
        
        # 有 sandbox_id 属性
        sandbox2 = Mock()
        del sandbox2.id
        sandbox2.sandbox_id = "sandbox-with-sandbox-id"
        
        session2 = SandboxSession(sample_template.template_id, sandbox2, Mock())
        assert session2.sandbox_id == "sandbox-with-sandbox-id"
        
        # 都没有，使用 fallback
        sandbox3 = Mock(spec=[])  # 空 spec 确保没有预定义属性
        
        session3 = SandboxSession(sample_template.template_id, sandbox3, Mock())
        assert session3.sandbox_id.startswith("sandbox-")
    
    @pytest.mark.unit
    def test_time_properties(self, mock_sandbox, sample_template):
        """测试时间相关属性"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        # 刚创建时
        assert session.age_seconds >= 0
        assert session.age_seconds < 1
        assert session.idle_seconds >= 0
        assert session.idle_seconds < 1
        
        # 更新活动时间
        import time
        time.sleep(0.01)
        session.last_activity = datetime.now()
        
        assert session.age_seconds > 0
        assert session.idle_seconds >= 0


class TestSandboxSessionIntegration:
    """SandboxSession 集成测试"""
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_session_lifecycle_complete(self, sample_template):
        """测试完整的会话生命周期"""
        sandbox = create_healthy_sandbox()
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=sandbox,
            client=mock_client
        )
        
        # 1. 初始状态
        assert session.status == SessionStatus.ACTIVE
        assert session.is_active
        
        # 2. 执行调用
        mock_http_client = AsyncMock()
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {"result": "success"}
        mock_http_client.post.return_value = mock_response
        
        with patch.object(session, '_get_http_client', return_value=mock_http_client):
            result = await session.invoke("test")
            assert result["result"] == "success"
        
        # 3. 暂停
        await session.pause()
        assert session.status == SessionStatus.PAUSED
        assert session.is_paused
        
        # 4. 恢复
        await session.resume()
        assert session.status == SessionStatus.ACTIVE
        assert session.is_active
        
        # 5. 关闭
        await session.close()
        assert session.status == SessionStatus.CLOSED
        assert not session.is_active
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_concurrent_operations(self, mock_sandbox, sample_template):
        """测试并发操作"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        # Mock HTTP 客户端
        mock_http_client = AsyncMock()
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {"result": "concurrent"}
        mock_http_client.post.return_value = mock_response
        mock_http_client.get.return_value = mock_response
        
        with patch.object(session, '_get_http_client', return_value=mock_http_client):
            # 并发执行多个操作
            tasks = [
                session.invoke(f"request-{i}")
                for i in range(5)
            ]
            tasks.append(session.ping())
            tasks.append(session.refresh())
            
            results = await asyncio.gather(*tasks, return_exceptions=True)
            
            # 验证结果
            assert len(results) == 7
            for i in range(5):
                assert results[i]["result"] == "concurrent"
    
    @pytest.mark.unit
    def test_session_representation(self, mock_sandbox, sample_template):
        """测试会话字符串表示"""
        mock_client = Mock()
        session = SandboxSession(
            template_id=sample_template.template_id,
            sandbox=mock_sandbox,
            client=mock_client
        )
        
        repr_str = repr(session)
        assert "SandboxSession" in repr_str
        assert session.sandbox_id in repr_str
        assert str(session.status) in repr_str  # SessionStatus.ACTIVE 而不是 'active'
        assert session.template_id in repr_str



"""
TemplateManager 单元测试

测试模板管理器的功能
"""

import pytest
from unittest.mock import AsyncMock, Mock, patch
from datetime import datetime

from ppio_sandbox.agent_runtime.client.template import TemplateManager
from ppio_sandbox.agent_runtime.client.auth import AuthManager
from ppio_sandbox.agent_runtime.client.exceptions import (
    TemplateNotFoundError,
    NetworkError,
    AuthenticationError
)
from ppio_sandbox.agent_runtime.client.models import AgentTemplate

from ..mocks.mock_api import (
    create_success_mock_client,
    create_auth_error_mock_client,
    create_template_not_found_mock_client,
    create_network_error_mock_client
)
from ..mocks.test_fixtures import create_sample_template, create_template_list


class TestTemplateManagerInit:
    """TemplateManager 初始化测试"""
    
    @pytest.mark.unit
    def test_template_manager_init(self, auth_manager: AuthManager):
        """测试模板管理器初始化"""
        manager = TemplateManager(auth_manager)
        
        assert manager.auth_manager is auth_manager
        assert manager.base_url == "https://api.ppio.ai"  # 默认 URL
        assert manager._client is None
    
    @pytest.mark.unit
    def test_template_manager_init_custom_url(self, auth_manager: AuthManager):
        """测试自定义 URL 初始化"""
        custom_url = "https://custom.api.com"
        manager = TemplateManager(auth_manager, base_url=custom_url)
        
        assert manager.base_url == custom_url
    
    @pytest.mark.unit
    def test_template_manager_init_env_url(self, auth_manager: AuthManager):
        """测试从环境变量读取 URL"""
        env_url = "https://env.api.com"
        with patch.dict('os.environ', {'PPIO_API_BASE_URL': env_url}):
            manager = TemplateManager(auth_manager)
            assert manager.base_url == env_url


class TestTemplateManagerHTTPClient:
    """TemplateManager HTTP 客户端测试"""
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_get_client_creation(self, auth_manager: AuthManager):
        """测试 HTTP 客户端创建"""
        manager = TemplateManager(auth_manager)
        
        client = await manager._get_client()
        
        assert client is not None
        assert manager._client is client
        
        # 第二次调用应该返回同一个客户端
        client2 = await manager._get_client()
        assert client2 is client
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_close_client(self, auth_manager: AuthManager):
        """测试关闭 HTTP 客户端"""
        manager = TemplateManager(auth_manager)
        
        # 创建客户端
        await manager._get_client()
        assert manager._client is not None
        
        # 关闭客户端
        await manager.close()
        assert manager._client is None


class TestTemplateManagerListTemplates:
    """TemplateManager 列出模板测试"""
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_list_templates_success(self, auth_manager: AuthManager):
        """测试成功列出模板"""
        manager = TemplateManager(auth_manager)
        mock_client = create_success_mock_client()
        
        with patch.object(manager, '_get_client', return_value=mock_client):
            templates = await manager.list_templates()
        
        assert isinstance(templates, list)
        assert len(templates) > 0
        assert all(isinstance(t, AgentTemplate) for t in templates)
        
        # 验证 HTTP 请求
        assert mock_client.get.called
        call_args = mock_client.get.call_args
        assert "/v1/templates/agents" in call_args[0][0]
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_list_templates_with_tags_filter(self, auth_manager: AuthManager):
        """测试按标签过滤模板"""
        manager = TemplateManager(auth_manager)
        mock_client = create_success_mock_client()
        
        with patch.object(manager, '_get_client', return_value=mock_client):
            await manager.list_templates(tags=["ai", "chat"])
        
        # 验证请求参数
        call_args = mock_client.get.call_args
        assert call_args[1]['params']['tags'] == "ai,chat"
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_list_templates_with_name_filter(self, auth_manager: AuthManager):
        """测试按名称过滤模板"""
        manager = TemplateManager(auth_manager)
        mock_client = create_success_mock_client()
        
        with patch.object(manager, '_get_client', return_value=mock_client):
            await manager.list_templates(name_filter="test-agent")
        
        # 验证请求参数
        call_args = mock_client.get.call_args
        assert call_args[1]['params']['name'] == "test-agent"
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_list_templates_auth_error(self, auth_manager: AuthManager):
        """测试认证错误"""
        manager = TemplateManager(auth_manager)
        mock_client = create_auth_error_mock_client()
        
        with patch.object(manager, '_get_client', return_value=mock_client):
            with pytest.raises(AuthenticationError) as exc_info:
                await manager.list_templates()
        
        assert "Invalid or expired API Key" in str(exc_info.value)
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_list_templates_network_error(self, auth_manager: AuthManager):
        """测试网络错误"""
        manager = TemplateManager(auth_manager)
        mock_client = create_network_error_mock_client()
        
        with patch.object(manager, '_get_client', return_value=mock_client):
            with pytest.raises(NetworkError) as exc_info:
                await manager.list_templates()
        
        assert "Failed to list templates" in str(exc_info.value)
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_list_templates_invalid_response(self, auth_manager: AuthManager):
        """测试无效响应处理"""
        manager = TemplateManager(auth_manager)
        mock_client = Mock()
        
        # 模拟返回无效的模板数据
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
            "templates": [
                {
                    "id": "invalid-template",
                    # 缺少必需字段
                }
            ]
        }
        mock_client.get = AsyncMock(return_value=mock_response)
        
        with patch.object(manager, '_get_client', return_value=mock_client):
            templates = await manager.list_templates()
        
        # 应该跳过无效模板，不抛出异常
        assert isinstance(templates, list)


class TestTemplateManagerGetTemplate:
    """TemplateManager 获取单个模板测试"""
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_get_template_success(self, auth_manager: AuthManager):
        """测试成功获取模板"""
        manager = TemplateManager(auth_manager)
        mock_client = create_success_mock_client()
        template_id = "test-template-123"
        
        with patch.object(manager, '_get_client', return_value=mock_client):
            template = await manager.get_template(template_id)
        
        assert isinstance(template, AgentTemplate)
        assert template.template_id == template_id
        
        # 验证 HTTP 请求
        call_args = mock_client.get.call_args
        assert f"/v1/templates/agents/{template_id}" in call_args[0][0]
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_get_template_not_found(self, auth_manager: AuthManager):
        """测试模板不存在"""
        manager = TemplateManager(auth_manager)
        mock_client = create_template_not_found_mock_client()
        template_id = "non-existent-template"
        
        with patch.object(manager, '_get_client', return_value=mock_client):
            with pytest.raises(TemplateNotFoundError) as exc_info:
                await manager.get_template(template_id)
        
        assert template_id in str(exc_info.value)
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_get_template_auth_error(self, auth_manager: AuthManager):
        """测试获取模板时的认证错误"""
        manager = TemplateManager(auth_manager)
        mock_client = create_auth_error_mock_client()
        
        with patch.object(manager, '_get_client', return_value=mock_client):
            with pytest.raises(AuthenticationError):
                await manager.get_template("any-template")
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_get_template_network_error(self, auth_manager: AuthManager):
        """测试获取模板时的网络错误"""
        manager = TemplateManager(auth_manager)
        mock_client = create_network_error_mock_client()
        
        with patch.object(manager, '_get_client', return_value=mock_client):
            with pytest.raises(NetworkError):
                await manager.get_template("any-template")


class TestTemplateManagerTemplateExists:
    """TemplateManager 模板存在检查测试"""
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_template_exists_true(self, auth_manager: AuthManager):
        """测试模板存在"""
        manager = TemplateManager(auth_manager)
        
        with patch.object(manager, 'get_template', return_value=create_sample_template()):
            exists = await manager.template_exists("existing-template")
        
        assert exists is True
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_template_exists_false(self, auth_manager: AuthManager):
        """测试模板不存在"""
        manager = TemplateManager(auth_manager)
        
        with patch.object(manager, 'get_template', side_effect=TemplateNotFoundError("Not found")):
            exists = await manager.template_exists("non-existing-template")
        
        assert exists is False
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_template_exists_error_handling(self, auth_manager: AuthManager):
        """测试其他错误时的处理"""
        manager = TemplateManager(auth_manager)
        
        # 网络错误等其他异常应该返回 False
        with patch.object(manager, 'get_template', side_effect=NetworkError("Network error")):
            exists = await manager.template_exists("any-template")
        
        assert exists is False


class TestTemplateManagerIntegration:
    """TemplateManager 集成测试"""
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_template_manager_lifecycle(self, auth_manager: AuthManager):
        """测试模板管理器完整生命周期"""
        manager = TemplateManager(auth_manager)
        
        try:
            # 1. 确保初始状态
            assert manager._client is None
            
            # 2. 模拟列出模板（会创建客户端）
            with patch.object(manager, '_get_client') as mock_get_client:
                mock_client = create_success_mock_client()
                mock_get_client.return_value = mock_client
                
                templates = await manager.list_templates()
                assert isinstance(templates, list)
                assert mock_get_client.called
            
            # 3. 模拟获取特定模板
            with patch.object(manager, '_get_client') as mock_get_client:
                mock_client = create_success_mock_client()
                mock_get_client.return_value = mock_client
                
                template = await manager.get_template("test-template-123")
                assert isinstance(template, AgentTemplate)
            
            # 4. 检查模板存在性
            with patch.object(manager, 'get_template', return_value=create_sample_template()):
                exists = await manager.template_exists("test-template-123")
                assert exists is True
        
        finally:
            # 5. 清理
            await manager.close()
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_concurrent_requests(self, auth_manager: AuthManager):
        """测试并发请求"""
        import asyncio
        
        manager = TemplateManager(auth_manager)
        mock_client = create_success_mock_client()
        
        with patch.object(manager, '_get_client', return_value=mock_client):
            # 并发执行多个请求
            tasks = [
                manager.list_templates(),
                manager.get_template("template-1"),
                manager.get_template("template-2"),
                manager.template_exists("template-3")
            ]
            
            results = await asyncio.gather(*tasks, return_exceptions=True)
            
            # 验证结果
            assert len(results) == 4
            assert isinstance(results[0], list)  # list_templates
            assert isinstance(results[1], AgentTemplate)  # get_template
            assert isinstance(results[2], AgentTemplate)  # get_template
            assert isinstance(results[3], bool)  # template_exists
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_error_recovery(self, auth_manager: AuthManager):
        """测试错误恢复"""
        manager = TemplateManager(auth_manager)
        
        # 第一次请求失败
        error_client = create_network_error_mock_client()
        with patch.object(manager, '_get_client', return_value=error_client):
            with pytest.raises(NetworkError):
                await manager.list_templates()
        
        # 第二次请求成功
        success_client = create_success_mock_client()
        with patch.object(manager, '_get_client', return_value=success_client):
            templates = await manager.list_templates()
            assert isinstance(templates, list)
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_request_parameters_validation(self, auth_manager: AuthManager):
        """测试请求参数验证"""
        manager = TemplateManager(auth_manager)
        mock_client = create_success_mock_client()
        
        with patch.object(manager, '_get_client', return_value=mock_client):
            # 测试各种参数组合
            await manager.list_templates()  # 无参数
            await manager.list_templates(tags=["ai"])  # 仅标签
            await manager.list_templates(name_filter="test")  # 仅名称
            await manager.list_templates(tags=["ai", "chat"], name_filter="test")  # 组合
        
        # 验证所有请求都被正确执行
        assert mock_client.get.call_count == 4
    
    @pytest.mark.unit
    @pytest.mark.asyncio
    @pytest.mark.asyncio

    async def test_template_data_consistency(self, auth_manager: AuthManager):
        """测试模板数据一致性"""
        manager = TemplateManager(auth_manager)
        mock_client = Mock()
        
        # 模拟一致的模板数据
        template_data = {
            "id": "consistent-template",
            "name": "consistent-agent",
            "version": "1.0.0",
            "description": "Consistent test template",
            "author": "test@example.com",
            "tags": ["test"],
            "created_at": "2024-01-01T00:00:00Z",
            "updated_at": "2024-01-01T00:00:00Z",
            "status": "active",
            "metadata": {},
            "size": 1024,
            "build_time": 30.0
        }
        
        # 列表响应
        list_response = Mock()
        list_response.status_code = 200
        list_response.json.return_value = {"templates": [template_data]}
        
        # 单个模板响应
        single_response = Mock()
        single_response.status_code = 200
        single_response.json.return_value = template_data
        
        mock_client.get = AsyncMock(side_effect=[list_response, single_response])
        
        with patch.object(manager, '_get_client', return_value=mock_client):
            # 从列表获取
            templates = await manager.list_templates()
            template_from_list = templates[0]
            
            # 直接获取
            template_direct = await manager.get_template("consistent-template")
            
            # 验证数据一致性
            assert template_from_list.template_id == template_direct.template_id
            assert template_from_list.name == template_direct.name
            assert template_from_list.version == template_direct.version
            assert template_from_list.author == template_direct.author



"""
模板管理器

管理 Agent 模板的查询，专注核心功能
"""

import asyncio
import os
from datetime import datetime
from typing import List, Optional
import httpx
import json

from .auth import AuthManager
from .exceptions import TemplateNotFoundError, NetworkError, AuthenticationError
from .models import AgentTemplate


class TemplateManager:
    """模板管理器 - 简化版本"""
    
    def __init__(self, auth_manager: AuthManager, base_url: Optional[str] = None):
        """初始化模板管理器
        
        Args:
            auth_manager: 认证管理器
            base_url: API 基础 URL
        """
        self.auth_manager = auth_manager
        self.base_url = base_url or os.getenv("PPIO_API_BASE_URL", "https://api.ppio.ai")
        self._client = None
    
    async def _get_client(self) -> httpx.AsyncClient:
        """获取 HTTP 客户端"""
        if self._client is None:
            self._client = httpx.AsyncClient(
                timeout=httpx.Timeout(30.0),
                headers=self.auth_manager.get_auth_headers()
            )
        return self._client
    
    async def close(self):
        """关闭 HTTP 客户端"""
        if self._client:
            await self._client.aclose()
            self._client = None
    
    async def list_templates(
        self, 
        tags: Optional[List[str]] = None,
        name_filter: Optional[str] = None
    ) -> List[AgentTemplate]:
        """列出模板
        
        Args:
            tags: 标签过滤
            name_filter: 名称过滤
            
        Returns:
            模板列表，每个模板的 metadata 字段包含 Agent 元信息
        """
        try:
            client = await self._get_client()
            
            # 构建查询参数
            params = {}
            if tags:
                params["tags"] = ",".join(tags)
            if name_filter:
                params["name"] = name_filter
            
            response = await client.get(
                f"{self.base_url}/v1/templates/agents",
                params=params
            )
            
            if response.status_code == 401:
                raise AuthenticationError("Invalid or expired API Key")
            elif response.status_code != 200:
                raise NetworkError(f"Failed to list templates: {response.status_code} {response.text}")
            
            data = response.json()
            templates = []
            
            for template_data in data.get("templates", []):
                try:
                    template = AgentTemplate(
                        template_id=template_data["id"],
                        name=template_data.get("name", "Unknown"),
                        version=template_data.get("version", "1.0.0"),
                        description=template_data.get("description"),
                        author=template_data.get("author"),
                        tags=template_data.get("tags", []),
                        created_at=datetime.fromisoformat(
                            template_data.get("created_at", datetime.now().isoformat())
                        ),
                        updated_at=datetime.fromisoformat(
                            template_data.get("updated_at", datetime.now().isoformat())
                        ),
                        status=template_data.get("status", "active"),
                        metadata=template_data.get("metadata", {}),
                        size=template_data.get("size"),
                        build_time=template_data.get("build_time"),
                        dependencies=template_data.get("dependencies", []),
                        runtime_info=template_data.get("runtime_info")
                    )
                    templates.append(template)
                except Exception as e:
                    # 跳过无效的模板数据，记录错误但不中断处理
                    print(f"Warning: Failed to parse template data: {e}")
                    continue
            
            return templates
            
        except AuthenticationError:
            raise
        except NetworkError:
            raise
        except Exception as e:
            raise NetworkError(f"Failed to list templates: {str(e)}")
    
    async def get_template(self, template_id: str) -> AgentTemplate:
        """获取特定模板
        
        Args:
            template_id: 模板 ID
            
        Returns:
            模板对象，包含完整的 Agent 元信息
            
        Raises:
            TemplateNotFoundError: 模板不存在时抛出
        """
        try:
            client = await self._get_client()
            
            response = await client.get(
                f"{self.base_url}/v1/templates/agents/{template_id}"
            )
            
            if response.status_code == 401:
                raise AuthenticationError("Invalid or expired API Key")
            elif response.status_code == 404:
                raise TemplateNotFoundError(f"Template {template_id} not found")
            elif response.status_code != 200:
                raise NetworkError(f"Failed to get template: {response.status_code} {response.text}")
            
            template_data = response.json()
            
            return AgentTemplate(
                template_id=template_data["id"],
                name=template_data.get("name", "Unknown"),
                version=template_data.get("version", "1.0.0"),
                description=template_data.get("description"),
                author=template_data.get("author"),
                tags=template_data.get("tags", []),
                created_at=datetime.fromisoformat(
                    template_data.get("created_at", datetime.now().isoformat())
                ),
                updated_at=datetime.fromisoformat(
                    template_data.get("updated_at", datetime.now().isoformat())
                ),
                status=template_data.get("status", "active"),
                metadata=template_data.get("metadata", {}),
                size=template_data.get("size"),
                build_time=template_data.get("build_time"),
                dependencies=template_data.get("dependencies", []),
                runtime_info=template_data.get("runtime_info")
            )
            
        except AuthenticationError:
            raise
        except TemplateNotFoundError:
            raise
        except NetworkError:
            raise
        except Exception as e:
            raise NetworkError(f"Failed to get template: {str(e)}")
    
    async def template_exists(self, template_id: str) -> bool:
        """检查模板是否存在
        
        Args:
            template_id: 模板 ID
            
        Returns:
            模板是否存在
        """
        try:
            await self.get_template(template_id)
            return True
        except TemplateNotFoundError:
            return False
        except Exception:
            # 其他错误（如网络错误）时，保守地返回 False
            return False